@@ -1,10 +1,11 @@
-/* Copyright (C) 1999-2003, Ghostgum Software Pty Ltd.  All rights reserved.
+/* Copyright (C) 2001-2006 artofcode LLC.
+   All Rights Reserved.
   
   This file is part of GNU ghostscript
 
-  GNU ghostscript is free software; you can redistribute it and/or modify it under
-  the terms of the GNU General Public License as published by the Free Software
-  Foundation; either version 2, or (at your option) any later version.
+  GNU ghostscript is free software; you can redistribute it and/or
+  modify it under the terms of the version 2 of the GNU General Public
+  License as published by the Free Software Foundation.
 
   GNU ghostscript is distributed in the hope that it will be useful, but WITHOUT
   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
@@ -16,7 +17,7 @@
 
 */
 
-// $Id: dwsetup.cpp,v 1.8 2006/06/16 12:55:04 Arabidopsis Exp $
+// $Id: dwsetup.cpp,v 1.10 2007/08/01 14:25:44 jemarch Exp $
 //
 //
 // This is the setup program for Win32 GNU Ghostscript
@@ -39,10 +40,10 @@
 // uninstall log files are to be placed.  
 // Subsequent lines contain files to be copied (but not directories).
 // For example, filelist.txt might contain:
-//   GNU Ghostscript 8.16
-//   gs8.16
-//   gs8.16\bin\gsdll32.dll
-//   gs8.16\lib\gs_init.ps
+//   GNU Ghostscript 8.55
+//   gs8.55
+//   gs8.55\bin\gsdll32.dll
+//   gs8.55\lib\gs_init.ps
 // The file fontlist.txt might contain:
 //   GNU Ghostscript Fonts
 //   fonts
@@ -719,7 +720,7 @@ install_all()
 BOOL
 install_prog()
 {
-	char *regkey1 = "AFPL Ghostscript";
+	char *regkey1 = "GPL Ghostscript";
 	char regkey2[16];
 	char szDLL[MAXSTR];
 	char szLIB[MAXSTR+MAXSTR];
@@ -1187,7 +1188,7 @@ BOOL make_filelist(int argc, char *argv[])
 		    if ((title == NULL) || (strlen(title) == 0) ||
 			(dir == NULL) || (strlen(dir) == 0) ||
 			(list == NULL) || (strlen(list) == 0)) {
-			message_box("Usage: setupgs -title \042GNU Ghostscript #.##\042 -dir \042gs#.##\042 -list \042filelist.txt\042 spec1 spec2 specn\n");
+			message_box("Usage: setupgs -title \042GPL Ghostscript #.##\042 -dir \042gs#.##\042 -list \042filelist.txt\042 spec1 spec2 specn\n");
 			return FALSE;
 		    }
 		    if (fList == (FILE *)NULL) {
