@@ -1,10 +1,11 @@
-/* Copyright (C) 2001-2004, Ghostgum Software Pty Ltd.  All rights reserved.
-
+/* Copyright (C) 2001-2006 artofcode LLC.
+   All Rights Reserved.
+  
   This file is part of GNU ghostscript
 
-  GNU ghostscript is free software; you can redistribute it and/or modify it under
-  the terms of the GNU General Public License as published by the Free Software
-  Foundation; either version 2, or (at your option) any later version.
+  GNU ghostscript is free software; you can redistribute it and/or
+  modify it under the terms of the version 2 of the GNU General Public
+  License as published by the Free Software Foundation.
 
   GNU ghostscript is distributed in the hope that it will be useful, but WITHOUT
   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
@@ -15,9 +16,8 @@
   Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 
 */
-
 /* gdevdsp.c */
-/* $Id: gdevdsp.c,v 1.7 2006/06/16 12:55:03 Arabidopsis Exp $ */
+/* $Id: gdevdsp.c,v 1.9 2007/08/01 14:25:47 jemarch Exp $ */
 
 /*
  * DLL based display device driver.
@@ -214,7 +214,7 @@ const gx_device_display gs_display_device =
       4,                        /* Number of colorants for CMYK */
       0,                        /* MaxSeparations has not been specified */
       {0},                      /* SeparationNames */
-      0,                        /* SeparationOrder */
+      {0},                      /* SeparationOrder names */
       {0, 1, 2, 3, 4, 5, 6, 7 } /* Initial component SeparationOrder */
     },
     { true }                   /* equivalent CMYK colors for spot colors */
