@@ -1,10 +1,11 @@
-/* Copyright (C) 1999, 2000, 2001 Aladdin Enterprises.  All rights reserved.
+/* Copyright (C) 2001-2006 artofcode LLC.
+   All Rights Reserved.
   
   This file is part of GNU ghostscript
 
-  GNU ghostscript is free software; you can redistribute it and/or modify it under
-  the terms of the GNU General Public License as published by the Free Software
-  Foundation; either version 2, or (at your option) any later version.
+  GNU ghostscript is free software; you can redistribute it and/or
+  modify it under the terms of the version 2 of the GNU General Public
+  License as published by the Free Software Foundation.
 
   GNU ghostscript is distributed in the hope that it will be useful, but WITHOUT
   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
@@ -16,7 +17,7 @@
 
 */
 
-/* $Id: gdevpdfu.c,v 1.7 2006/06/16 12:55:04 Arabidopsis Exp $ */
+/* $Id: gdevpdfu.c,v 1.10 2007/08/01 14:25:52 jemarch Exp $ */
 /* Output utilities for PDF-writing driver */
 #include "memory_.h"
 #include "jpeglib_.h"		/* for sdct.h */
@@ -241,7 +242,7 @@ copy_procsets(stream *s, const gs_param_string *path, bool HaveTrueTypes)
     if (p != NULL) {
 	for (;; i++) {
 	    const byte *c = memchr(p, gp_file_name_list_separator, e - p);
-	    int k;
+	    int k = 0; /* Initializing against a compiler warning only. */
 
 	    if (c == NULL)
 		c = e;
@@ -333,7 +334,6 @@ pdf_open_document(gx_device_pdf * pdev)
 	    pdev->OPDFRead_procset_length = stell(s);
 	}
 	pprintd2(s, "%%PDF-%d.%d\n", level / 10, level % 10);
-	pdev->binary_ok = !pdev->params.ASCII85EncodePages;
 	if (pdev->binary_ok)
 	    stream_puts(s, "%\307\354\217\242\n");
     }
@@ -542,8 +542,12 @@ none_to_stream(gx_device_pdf * pdev)
 	pdev->contents_length_id = pdf_obj_ref(pdev);
 	s = pdev->strm;
 	pprintld1(s, "<</Length %ld 0 R", pdev->contents_length_id);
-	if (pdev->compression == pdf_compress_Flate)
-	    pprints1(s, "/Filter /%s", compression_filter_name);
+	if (pdev->compression == pdf_compress_Flate) {
+	    if (pdev->binary_ok)
+		pprints1(s, "/Filter /%s", compression_filter_name);
+	    else 
+		pprints1(s, "/Filter [/ASCII85Decode /%s]", compression_filter_name);
+	}
 	stream_puts(s, ">>\nstream\n");
 	pdev->contents_pos = pdf_stell(pdev);
 	code = pdf_begin_encrypt(pdev, &s, pdev->contents_id);
@@ -551,14 +555,36 @@ none_to_stream(gx_device_pdf * pdev)
 	    return code;
 	pdev->strm = s;
 	if (pdev->compression == pdf_compress_Flate) {	/* Set up the Flate filter. */
-	    const stream_template *template = &compression_filter_template;
-	    stream *es = s_alloc(pdev->pdf_memory, "PDF compression stream");
-	    byte *buf = gs_alloc_bytes(pdev->pdf_memory, sbuf_size,
-				       "PDF compression buffer");
-	    compression_filter_state *st =
-		gs_alloc_struct(pdev->pdf_memory, compression_filter_state,
-				template->stype, "PDF compression state");
+	    const stream_template *template;
+	    stream *es;
+	    byte *buf;
+	    compression_filter_state *st;
 
+	    if (!pdev->binary_ok) {	/* Set up the A85 filter */
+		const stream_template *template = &s_A85E_template;
+		stream *as = s_alloc(pdev->pdf_memory, "PDF contents stream");
+		byte *buf = gs_alloc_bytes(pdev->pdf_memory, sbuf_size,
+					   "PDF contents buffer");
+		stream_A85E_state *ast = gs_alloc_struct(pdev->pdf_memory, stream_A85E_state,
+				template->stype, "PDF contents state");
+		if (as == 0 || ast == 0 || buf == 0)
+		    return_error(gs_error_VMerror);
+		s_std_init(as, buf, sbuf_size, &s_filter_write_procs,
+			   s_mode_write);
+		ast->memory = pdev->pdf_memory;
+		ast->template = template;
+		as->state = (stream_state *) ast;
+		as->procs.process = template->process;
+		as->strm = s;
+		(*template->init) ((stream_state *) ast);
+		pdev->strm = s = as;
+	    }
+	    template = &compression_filter_template;
+	    es = s_alloc(pdev->pdf_memory, "PDF compression stream");
+	    buf = gs_alloc_bytes(pdev->pdf_memory, sbuf_size,
+				       "PDF compression buffer");
+	    st = gs_alloc_struct(pdev->pdf_memory, compression_filter_state,
+				template->stype, "PDF compression state");
 	    if (es == 0 || st == 0 || buf == 0)
 		return_error(gs_error_VMerror);
 	    s_std_init(es, buf, sbuf_size, &s_filter_write_procs,
@@ -651,10 +677,17 @@ stream_to_none(gx_device_pdf * pdev)
     } else {
 	if (pdev->vgstack_depth)
 	    pdf_restore_viewer_state(pdev, s);
-	if (pdev->compression_at_page_start == pdf_compress_Flate) {	/* Terminate the Flate filter. */
+	if (pdev->compression_at_page_start == pdf_compress_Flate) {	/* Terminate the filters. */
 	    stream *fs = s->strm;
 
-	    sclose(s);
+	    if (!pdev->binary_ok) {
+		sclose(s);	/* Terminate the ASCII85 filter. */
+		gs_free_object(pdev->pdf_memory, s->cbuf, "A85E contents buffer");
+		gs_free_object(pdev->pdf_memory, s, "A85E contents stream");
+		pdev->strm = s = fs;
+		fs = s->strm;
+	    }
+	    sclose(s);		/* Next terminate the compression filter */
 	    gs_free_object(pdev->pdf_memory, s->cbuf, "zlib buffer");
 	    gs_free_object(pdev->pdf_memory, s, "zlib stream");
 	    pdev->strm = s = fs;
@@ -1148,7 +1181,8 @@ pdf_write_and_free_all_resource_objects(gx_device_pdf *pdev)
 	if (code >= 0)
 	    code = code1;
     }
-    code1 = pdf_finish_font_descriptors(pdev, pdf_release_FontDescriptor_components);
+    code1 = pdf_finish_resources(pdev, resourceFontDescriptor,
+			pdf_release_FontDescriptor_components);
     if (code >= 0)
 	code = code1;
     for (i = 0; i < NUM_RESOURCE_TYPES; ++i) {
