@@ -1,10 +1,11 @@
-/* Copyright (C) 2002 Aladdin Enterprises.  All rights reserved.
+/* Copyright (C) 2001-2006 artofcode LLC.
+   All Rights Reserved.
   
   This file is part of GNU ghostscript
 
-  GNU ghostscript is free software; you can redistribute it and/or modify it under
-  the terms of the GNU General Public License as published by the Free Software
-  Foundation; either version 2, or (at your option) any later version.
+  GNU ghostscript is free software; you can redistribute it and/or
+  modify it under the terms of the version 2 of the GNU General Public
+  License as published by the Free Software Foundation.
 
   GNU ghostscript is distributed in the hope that it will be useful, but WITHOUT
   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
@@ -16,11 +17,12 @@
 
 */
 
-/* $Id: gdevpdts.c,v 1.5 2006/06/16 12:55:04 Arabidopsis Exp $ */
+/* $Id: gdevpdts.c,v 1.7 2007/08/01 14:25:53 jemarch Exp $ */
 /* Text state management for pdfwrite */
 #include "math_.h"
 #include "memory_.h"
 #include "gx.h"
+#include "gserrors.h"
 #include "gdevpdfx.h"
 #include "gdevpdtx.h"
 #include "gdevpdtf.h"		/* for pdfont->FontType */
@@ -153,7 +155,15 @@ set_text_distance(gs_point *pdist, floatp dx, floatp dy, const gs_matrix *pmat)
     int code = gs_distance_transform_inverse(dx, dy, pmat, pdist);
     double rounded;
 
-    if (code < 0)
+    if (code == gs_error_undefinedresult) {
+	/* The CTM is degenerate.
+	   Can't know the distance in user space.
+	   Set zero because we believe it is not important for rendering.
+	   We want to copy the text to PDF to make it searchable.
+	   Bug 689006.
+	 */
+	pdist->x = pdist->y = 0;
+    } else if (code < 0)
 	return code;
     /* If the distance is very close to integers, round it. */
     if (fabs(pdist->x - (rounded = floor(pdist->x + 0.5))) < 0.0005)
