@@ -1,10 +1,11 @@
-/* Copyright (C) 1990, 2000 Aladdin Enterprises.  All rights reserved.
+/* Copyright (C) 2001-2006 artofcode LLC.
+   All Rights Reserved.
   
   This file is part of GNU ghostscript
 
-  GNU ghostscript is free software; you can redistribute it and/or modify it under
-  the terms of the GNU General Public License as published by the Free Software
-  Foundation; either version 2, or (at your option) any later version.
+  GNU ghostscript is free software; you can redistribute it and/or
+  modify it under the terms of the version 2 of the GNU General Public
+  License as published by the Free Software Foundation.
 
   GNU ghostscript is distributed in the hope that it will be useful, but WITHOUT
   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
@@ -16,7 +17,7 @@
 
 */
 
-/* $Id: gdevprn.c,v 1.7 2006/06/16 12:55:04 Arabidopsis Exp $ */
+/* $Id: gdevprn.c,v 1.9 2007/08/01 14:25:54 jemarch Exp $ */
 /* Generic printer driver support */
 #include "ctype_.h"
 #include "gdevprn.h"
@@ -1044,7 +1045,8 @@ gx_default_create_buf_device(gx_device **pbdev, gx_device *target,
     } else
 	bdev = (gx_device *)mdev;
     /****** QUESTIONABLE, BUT BETTER THAN OMITTING ******/
-    bdev->color_info = target->color_info;
+    if (&bdev->color_info != &target->color_info) /* Pacify Valgrind */
+        bdev->color_info = target->color_info;
     *pbdev = bdev;
     return 0;
 }
@@ -1085,6 +1087,13 @@ gx_default_setup_buf_device(gx_device *bdev, byte *buffer, int bytes_per_line,
     if ((gx_device *)mdev == bdev && mdev->num_planes)
 	raster = bitmap_raster(mdev->planes[0].depth * mdev->width);
     if (ptrs == 0) {
+	/* 
+	 * Before allocating a new line pointer array, if there is a previous
+	 * array, free it to prevent leaks.
+	 */
+	if (mdev->line_ptrs != NULL)
+	    gs_free_object(mdev->line_pointer_memory, mdev->line_ptrs,
+		       "mem_close");
 	/*
 	 * Allocate line pointers now; free them when we close the device.
 	 * Note that for multi-planar devices, we have to allocate using
