@@ -1,10 +1,11 @@
-/* Copyright (C) 1997, 2000-2004 artofcode LLC. All rights reserved.
+/* Copyright (C) 2001-2006 artofcode LLC.
+   All Rights Reserved.
   
   This file is part of GNU ghostscript
 
-  GNU ghostscript is free software; you can redistribute it and/or modify it under
-  the terms of the GNU General Public License as published by the Free Software
-  Foundation; either version 2, or (at your option) any later version.
+  GNU ghostscript is free software; you can redistribute it and/or
+  modify it under the terms of the version 2 of the GNU General Public
+  License as published by the Free Software Foundation.
 
   GNU ghostscript is distributed in the hope that it will be useful, but WITHOUT
   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
@@ -16,7 +17,7 @@
 
 */
 
-/* $Id: gdevps.c,v 1.7 2006/06/16 12:55:03 Arabidopsis Exp $ */
+/* $Id: gdevps.c,v 1.9 2007/08/01 14:25:54 jemarch Exp $ */
 /* PostScript-writing driver */
 #include "math_.h"
 #include "memory_.h"
@@ -1453,7 +1454,7 @@ psw_begin_image(gx_device * dev,
 		) {
 		goto fail;
 	    }
-	    pbcs = (const gs_color_space *)&pcs->params.indexed.base_space;
+	    pbcs = pcs->base_space;
 	    switch (gs_color_space_get_index(pbcs)) {
 	    case gs_color_space_index_DeviceGray:
 		base_name = "DeviceGray"; break;
