@@ -1,10 +1,11 @@
-/* Copyright (C) 2003 artofcode LLC.  All rights reserved.
-
+/* Copyright (C) 2001-2006 artofcode LLC.
+   All Rights Reserved.
+  
   This file is part of GNU ghostscript
 
-  GNU ghostscript is free software; you can redistribute it and/or modify it under
-  the terms of the GNU General Public License as published by the Free Software
-  Foundation; either version 2, or (at your option) any later version.
+  GNU ghostscript is free software; you can redistribute it and/or
+  modify it under the terms of the version 2 of the GNU General Public
+  License as published by the Free Software Foundation.
 
   GNU ghostscript is distributed in the hope that it will be useful, but WITHOUT
   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
@@ -13,10 +14,10 @@
   You should have received a copy of the GNU General Public License along with
   ghostscript; see the file COPYING. If not, write to the Free Software Foundation,
   Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
-  
+
 */
 
-/*$Id: gdevrinkj.c,v 1.2 2006/06/16 18:54:58 Arabidopsis Exp $ */
+/*$Id: gdevrinkj.c,v 1.4 2007/08/01 14:25:55 jemarch Exp $ */
 /* Support for rinkj (resplendent inkjet) drivers. */
 
 #include "math_.h"
@@ -219,7 +220,6 @@ const rinkj_device gs_rinkj_device =
     /* DeviceN device specific parameters */
     RINKJ_DEVICE_CMYK,		/* Color model */
     8,				/* Bits per color - must match ncomp, depth, etc. above */
-    0,
     (&DeviceCMYKComponents),	/* Names of color model colorants */
     4,				/* Number colorants for CMYK */
     {0},			/* SeparationNames */
@@ -763,7 +763,7 @@ rinkj_put_params(gx_device * pdev, gs_param_list * plist)
  * number if the name is found.  It returns a negative value if not found.
  */
 private int
-rinkj_get_color_comp_index(gx_device * dev, const char * pname, int name_size, int src_index)
+rinkj_get_color_comp_index(const gx_device * dev, const char * pname, int name_size, int src_index)
 {
 /* TO_DO_DEVICEN  This routine needs to include the effects of the SeparationOrder array */
     const fixed_colorant_names_list * list = ((const rinkj_device *)dev)->std_colorant_names;
