@@ -1,10 +1,11 @@
-/* Copyright (C) 1989, 2000 Aladdin Enterprises.  All rights reserved.
+/* Copyright (C) 2001-2006 artofcode LLC.
+   All Rights Reserved.
   
   This file is part of GNU ghostscript
 
-  GNU ghostscript is free software; you can redistribute it and/or modify it under
-  the terms of the GNU General Public License as published by the Free Software
-  Foundation; either version 2, or (at your option) any later version.
+  GNU ghostscript is free software; you can redistribute it and/or
+  modify it under the terms of the version 2 of the GNU General Public
+  License as published by the Free Software Foundation.
 
   GNU ghostscript is distributed in the hope that it will be useful, but WITHOUT
   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
@@ -16,7 +17,7 @@
 
 */
 
-/* $Id: gdevwpr2.c,v 1.6 2006/03/08 12:30:24 Arabidopsis Exp $ */
+/* $Id: gdevwpr2.c,v 1.8 2007/08/01 14:25:57 jemarch Exp $ */
 /*
  * Microsoft Windows 3.n printer driver for Ghostscript.
  * Original version by Russell Lang and
@@ -397,12 +398,13 @@ win_pr2_open(gx_device * dev)
 
     /* gdev_prn_open opens a temporary file which we don't want */
     /* so we specify the name now so we can delete it later */
+    wdev->fname[0] = '\0';
     pfile = gp_open_scratch_file(gp_scratch_file_name_prefix,
 				 wdev->fname, "wb");
     fclose(pfile);
     code = gdev_prn_open(dev);
-    /* delete unwanted temporary file */
-    unlink(wdev->fname);
+    if ((code < 0) && wdev->fname[0])
+        unlink(wdev->fname);
 
     if (!wdev->nocancel) {
 	/* inform user of progress with dialog box and allow cancel */
@@ -449,6 +451,11 @@ win_pr2_close(gx_device * dev)
     }
 
     code = gdev_prn_close(dev);
+
+    /* delete unwanted temporary file */
+    if (wdev->fname[0])
+	unlink(wdev->fname);
+
     return code;
 }
 
@@ -868,7 +875,7 @@ win_pr2_getdc(gx_device_win_pr2 * wdev)
     char *output;
     char *devcap;
     int devcapsize;
-    int size;
+    int devmode_size;
 
     int i, n;
     POINT *pp;
@@ -878,11 +885,7 @@ win_pr2_getdc(gx_device_win_pr2 * wdev)
     int papersize;
     char papername[64];
     char drvname[32];
-    HINSTANCE hlib;
-    LPFNDEVMODE pfnExtDeviceMode;
-    LPFNDEVCAPS pfnDeviceCapabilities;
     LPDEVMODE podevmode, pidevmode;
-
     HANDLE hprinter;
 
     /* first try to derive the printer name from -sOutputFile= */
@@ -917,66 +920,28 @@ win_pr2_getdc(gx_device_win_pr2 * wdev)
     GetProfileString("Devices", device, "", driverbuf, sizeof(driverbuf));
     driver = strtok(driverbuf, ",");
     output = strtok(NULL, ",");
-    if (is_win32s)
-    {
-	strcpy(drvname, driver);
-	strcat(drvname, ".drv");
-	driver = drvname;
-    }
 
-    if (!is_win32s) {		/* Win32 */
-	if (!OpenPrinter(device, &hprinter, NULL))
-	    return FALSE;
-	size = DocumentProperties(NULL, hprinter, device, NULL, NULL, 0);
-	if ((podevmode = gs_malloc(wdev->memory, size, 1, "win_pr2_getdc")) == (LPDEVMODE) NULL) {
-	    ClosePrinter(hprinter);
-	    return FALSE;
-	}
-	if ((pidevmode = gs_malloc(wdev->memory, size, 1, "win_pr2_getdc")) == (LPDEVMODE) NULL) {
-	    gs_free(wdev->memory, podevmode, size, 1, "win_pr2_getdc");
-	    ClosePrinter(hprinter);
-	    return FALSE;
-	}
-	DocumentProperties(NULL, hprinter, device, podevmode, NULL, DM_OUT_BUFFER);
-	pfnDeviceCapabilities = (LPFNDEVCAPS) DeviceCapabilities;
-    } else
-    {				/* Win16 and Win32s */
-	/* now load the printer driver */
-	hlib = LoadLibrary(driver);
-	if (hlib < (HINSTANCE) HINSTANCE_ERROR)
-	    return FALSE;
-
-	/* call ExtDeviceMode() to get default parameters */
-	pfnExtDeviceMode = (LPFNDEVMODE) GetProcAddress(hlib, "ExtDeviceMode");
-	if (pfnExtDeviceMode == (LPFNDEVMODE) NULL) {
-	    FreeLibrary(hlib);
-	    return FALSE;
-	}
-	pfnDeviceCapabilities = (LPFNDEVCAPS) GetProcAddress(hlib, "DeviceCapabilities");
-	if (pfnDeviceCapabilities == (LPFNDEVCAPS) NULL) {
-	    FreeLibrary(hlib);
-	    return FALSE;
-	}
-	size = pfnExtDeviceMode(NULL, hlib, NULL, device, output, NULL, NULL, 0);
-	if ((podevmode = gs_malloc(wdev->memory, size, 1, "win_pr2_getdc")) == (LPDEVMODE) NULL) {
-	    FreeLibrary(hlib);
-	    return FALSE;
-	}
-	if ((pidevmode = gs_malloc(wdev->memory, size, 1, "win_pr2_getdc")) == (LPDEVMODE) NULL) {
-	    gs_free(wdev->memory, podevmode, size, 1, "win_pr2_getdc");
-	    FreeLibrary(hlib);
-	    return FALSE;
-	}
-	pfnExtDeviceMode(NULL, hlib, podevmode, device, output,
-			 NULL, NULL, DM_OUT_BUFFER);
+    if (!OpenPrinter(device, &hprinter, NULL))
+	return FALSE;
+    devmode_size = DocumentProperties(NULL, hprinter, device, NULL, NULL, 0);
+    if ((podevmode = gs_malloc(wdev->memory, devmode_size, 1, "win_pr2_getdc"))
+	== (LPDEVMODE) NULL) {
+	ClosePrinter(hprinter);
+	return FALSE;
     }
+    if ((pidevmode = gs_malloc(wdev->memory, devmode_size, 1, "win_pr2_getdc"))		== (LPDEVMODE) NULL) {
+	gs_free(wdev->memory, podevmode, devmode_size, 1, "win_pr2_getdc");
+	ClosePrinter(hprinter);
+	return FALSE;
+    }
+    DocumentProperties(NULL, hprinter, device, podevmode, NULL, DM_OUT_BUFFER);
 
     /* now find out what paper sizes are available */
-    devcapsize = pfnDeviceCapabilities(device, output, DC_PAPERSIZE, NULL, NULL);
+    devcapsize = DeviceCapabilities(device, output, DC_PAPERSIZE, NULL, NULL);
     devcapsize *= sizeof(POINT);
     if ((devcap = gs_malloc(wdev->memory, devcapsize, 1, "win_pr2_getdc")) == (LPBYTE) NULL)
 	return FALSE;
-    n = pfnDeviceCapabilities(device, output, DC_PAPERSIZE, devcap, NULL);
+    n = DeviceCapabilities(device, output, DC_PAPERSIZE, devcap, NULL);
     paperwidth = (int)(wdev->MediaSize[0] * 254 / 72);
     paperheight = (int)(wdev->MediaSize[1] * 254 / 72);
     papername[0] = '\0';
@@ -1011,26 +976,26 @@ win_pr2_getdc(gx_device_win_pr2 * wdev)
     gs_free(wdev->memory, devcap, devcapsize, 1, "win_pr2_getdc");
     
     /* get the dmPaperSize */
-    devcapsize = pfnDeviceCapabilities(device, output, DC_PAPERS, NULL, NULL);
+    devcapsize = DeviceCapabilities(device, output, DC_PAPERS, NULL, NULL);
     devcapsize *= sizeof(WORD);
     if ((devcap = gs_malloc(wdev->memory, devcapsize, 1, "win_pr2_getdc")) == (LPBYTE) NULL)
 	return FALSE;
-    n = pfnDeviceCapabilities(device, output, DC_PAPERS, devcap, NULL);
+    n = DeviceCapabilities(device, output, DC_PAPERS, devcap, NULL);
     if ((paperindex >= 0) && (paperindex < n))
 	papersize = ((WORD *) devcap)[paperindex];
     gs_free(wdev->memory, devcap, devcapsize, 1, "win_pr2_getdc");
 
     /* get the paper name */
-    devcapsize = pfnDeviceCapabilities(device, output, DC_PAPERNAMES, NULL, NULL);
+    devcapsize = DeviceCapabilities(device, output, DC_PAPERNAMES, NULL, NULL);
     devcapsize *= 64;
     if ((devcap = gs_malloc(wdev->memory, devcapsize, 1, "win_pr2_getdc")) == (LPBYTE) NULL)
 	return FALSE;
-    n = pfnDeviceCapabilities(device, output, DC_PAPERNAMES, devcap, NULL);
+    n = DeviceCapabilities(device, output, DC_PAPERNAMES, devcap, NULL);
     if ((paperindex >= 0) && (paperindex < n))
 	strcpy(papername, devcap + paperindex * 64);
     gs_free(wdev->memory, devcap, devcapsize, 1, "win_pr2_getdc");
 
-    memcpy(pidevmode, podevmode, size);
+    memcpy(pidevmode, podevmode, devmode_size);
 
     pidevmode->dmFields = 0;
     
@@ -1073,40 +1038,26 @@ win_pr2_getdc(gx_device_win_pr2 * wdev)
     
     win_pr2_update_win(wdev, pidevmode);
     
-    if (!is_win32s) {
-	
-	/* merge the entries */
-	DocumentProperties(NULL, hprinter, device, podevmode, pidevmode, DM_IN_BUFFER | DM_OUT_BUFFER);
-	ClosePrinter(hprinter);
-	
-	/* now get a DC */
-	wdev->hdcprn = CreateDC(driver, device, NULL, podevmode);
-    } else
-    {				/* Win16 and Win32s */
-	pfnExtDeviceMode(NULL, hlib, podevmode, device, output,
-			 pidevmode, NULL, DM_IN_BUFFER | DM_OUT_BUFFER);
-	/* release the printer driver */
-	FreeLibrary(hlib);
-	/* now get a DC */
-	if (is_win32s)
-	    strtok(driver, ".");	/* remove .drv */
-	wdev->hdcprn = CreateDC(driver, device, output, podevmode);
-    }
+    /* merge the entries */
+    DocumentProperties(NULL, hprinter, device, podevmode, pidevmode, DM_IN_BUFFER | DM_OUT_BUFFER);
+    ClosePrinter(hprinter);
     
-    if (wdev->win32_hdevmode == NULL) {
-	wdev->win32_hdevmode = GlobalAlloc(0, sizeof(DEVMODE));
-    }
+    /* now get a DC */
+    wdev->hdcprn = CreateDC(driver, device, NULL, podevmode);
+    
+    if (wdev->win32_hdevmode == NULL)
+	wdev->win32_hdevmode = GlobalAlloc(0, devmode_size);
     
     if (wdev->win32_hdevmode) {
 	LPDEVMODE pdevmode = (LPDEVMODE) GlobalLock(wdev->win32_hdevmode);
 	if (pdevmode) {
-	    memcpy(pdevmode, podevmode, sizeof(DEVMODE));
+	    memcpy(pdevmode, podevmode, devmode_size);
 	    GlobalUnlock(wdev->win32_hdevmode);
 	}
     }
 
-    gs_free(wdev->memory, pidevmode, size, 1, "win_pr2_getdc");
-    gs_free(wdev->memory, podevmode, size, 1, "win_pr2_getdc");
+    gs_free(wdev->memory, pidevmode, devmode_size, 1, "win_pr2_getdc");
+    gs_free(wdev->memory, podevmode, devmode_size, 1, "win_pr2_getdc");
 
     if (wdev->hdcprn != (HDC) NULL)
 	return TRUE;		/* success */
