@@ -1,10 +1,11 @@
-/* Copyright (C) 1989-2003 artofcode, LLC.  All rights reserved.
+/* Copyright (C) 2001-2006 artofcode LLC.
+   All Rights Reserved.
   
   This file is part of GNU ghostscript
 
-  GNU ghostscript is free software; you can redistribute it and/or modify it under
-  the terms of the GNU General Public License as published by the Free Software
-  Foundation; either version 2, or (at your option) any later version.
+  GNU ghostscript is free software; you can redistribute it and/or
+  modify it under the terms of the version 2 of the GNU General Public
+  License as published by the Free Software Foundation.
 
   GNU ghostscript is distributed in the hope that it will be useful, but WITHOUT
   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
@@ -16,7 +17,7 @@
 
 */
 
-/* $Id: gp_macio.c,v 1.6 2006/03/08 12:30:25 Arabidopsis Exp $ */
+/* $Id: gp_macio.c,v 1.8 2007/08/01 14:25:58 jemarch Exp $ */
 
 #ifndef __CARBON__
 //#include "MacHeaders"
@@ -1019,3 +1020,38 @@ int gp_enumerate_fonts_next(void *enum_state, char **fontname, char **path)
 	return 1;
 }
                                                                                 
+/* --------- 64 bit file access ----------- */
+/* fixme: Not implemented yet.
+ * Currently we stub it with 32 bits access. 
+ */
+
+FILE *gp_fopen_64(const char *filename, const char *mode)
+{
+    return fopen(filename, mode);
+}
+
+FILE *gp_open_scratch_file_64(const char *prefix,
+			   char fname[gp_file_name_sizeof],
+			   const char *mode)
+{
+    return gp_open_scratch_file(prefix, fname, mode);
+}
+
+FILE *gp_open_printer_64(char fname[gp_file_name_sizeof], int binary_mode)
+{
+    return gp_open_printer(fname, binary_mode);
+}
+
+int64_t gp_ftell_64(FILE *strm)
+{
+    return ftell(strm);
+}
+
+int gp_fseek_64(FILE *strm, int64_t offset, int origin)
+{
+    long offset1 = (long)offset;
+    
+    if (offset != offset1)
+	return -1;
+    return fseek(strm, offset1, origin);
+}
