@@ -1,10 +1,11 @@
-/* Copyright (C) 1997, 2001 Aladdin Enterprises.  All rights reserved.
+/* Copyright (C) 2001-2006 artofcode LLC.
+   All Rights Reserved.
   
   This file is part of GNU ghostscript
 
-  GNU ghostscript is free software; you can redistribute it and/or modify it under
-  the terms of the GNU General Public License as published by the Free Software
-  Foundation; either version 2, or (at your option) any later version.
+  GNU ghostscript is free software; you can redistribute it and/or
+  modify it under the terms of the version 2 of the GNU General Public
+  License as published by the Free Software Foundation.
 
   GNU ghostscript is distributed in the hope that it will be useful, but WITHOUT
   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
@@ -16,7 +17,7 @@
 
 */
 
-/* $Id: gp_mslib.c,v 1.5 2006/03/08 12:30:23 Arabidopsis Exp $ */
+/* $Id: gp_mslib.c,v 1.7 2007/08/01 14:25:59 jemarch Exp $ */
 /*
  * Microsoft Windows platform support for Graphics Library
  *
@@ -30,6 +31,9 @@
  * This is called frequently - it must be quick!
  */
 #ifdef CHECK_INTERRUPTS
+
+#include "gx.h"
+
 int
 gp_check_interrupts(const gs_memory_t *mem)
 {
