@@ -1,10 +1,11 @@
-/* Copyright (C) 1992, 2000-2003 artofcode LLC.  All rights reserved.
+/* Copyright (C) 2001-2006 artofcode LLC.
+   All Rights Reserved.
   
   This file is part of GNU ghostscript
 
-  GNU ghostscript is free software; you can redistribute it and/or modify it under
-  the terms of the GNU General Public License as published by the Free Software
-  Foundation; either version 2, or (at your option) any later version.
+  GNU ghostscript is free software; you can redistribute it and/or
+  modify it under the terms of the version 2 of the GNU General Public
+  License as published by the Free Software Foundation.
 
   GNU ghostscript is distributed in the hope that it will be useful, but WITHOUT
   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
@@ -16,7 +17,7 @@
 
 */
 
-/* $Id: gp_mswin.c,v 1.7 2006/06/16 12:55:03 Arabidopsis Exp $ */
+/* $Id: gp_mswin.c,v 1.9 2007/08/01 14:25:59 jemarch Exp $ */
 /*
  * Microsoft Windows platform support for Ghostscript.
  *
@@ -51,6 +52,7 @@
 #include "gsexit.h"
 
 #include "windows_.h"
+#include <stdarg.h>
 #include <shellapi.h>
 #include <winspool.h>
 #include "gp_mswin.h"
@@ -133,6 +135,8 @@ gp_open_printer(char fname[gp_file_name_sizeof], int binary_mode)
 	return pfile;
     } else if (fname[0] == '|') 	/* pipe */
 	return popen(fname + 1, (binary_mode ? "wb" : "w"));
+    else if (strcmp(fname, "LPT1:") == 0)
+	return NULL;	/* not supported, use %printer%name instead  */
     else
 	return fopen(fname, (binary_mode ? "wb" : "w"));
 }
@@ -212,11 +216,6 @@ is_printer(const char *name)
     if (strlen(name) == 0)
 	return TRUE;
 
-    /*  is printer if name appears in win.ini [ports] section */
-    GetProfileString("ports", name, "XYZ", buf, sizeof(buf));
-    if (strlen(name) == 0 || strcmp(buf, "XYZ"))
-	return TRUE;
-
     /* is printer if name prefixed by \\spool\ */
     if (is_spool(name))
 	return TRUE;
@@ -229,71 +228,43 @@ is_printer(const char *name)
 /* Print File to port or queue */
 /* port==NULL means prompt for port or queue with dialog box */
 
-/* This is messy because Microsoft changed the spooler interface */
-/* between Windows 3.1 and Windows 95/NT */
-/* and didn't provide the spooler interface in Win32s */
+/* This is messy because of past support for old version of Windows. */
 
 /* Win95, WinNT: Use OpenPrinter, WritePrinter etc. */
 private int gp_printfile_win32(const char *filename, char *port);
 
-/* Win32s: Pass to Win16 spooler via gs16spl.exe */
-private int gp_printfile_gs16spl(const char *filename, const char *port);
-
 /*
  * Valid values for pmport are:
  *   ""
- *      action: WinNT and Win95 use default queue, Win32s prompts for port
- *   "LPT1:" (or other port that appears in win.ini [ports]
- *      action: start gs16spl.exe to print to the port
+ *      action: Use default queue
  *   "\\spool\printer name"
- *      action: send to printer using WritePrinter (WinNT and Win95).
- *      action: translate to port name using win.ini [Devices]
- *              then use gs16spl.exe (Win32s).
+ *      action: send to printer using WritePrinter.
+ *              Using "%printer%printer name" is preferred
  *   "\\spool"
- *      action: prompt for queue name then send to printer using 
- *              WritePrinter (WinNT and Win95).
- *      action: prompt for port then use gs16spl.exe (Win32s).
+ *      action: prompt for queue name then send to printer using WritePrinter.
+ *              THIS IS CURRENTLY BROKEN
  */
 /* Print File */
 private int
 gp_printfile(const char *filename, const char *pmport)
 {
-    /* treat WinNT and Win95 differently to Win32s */
-    if (!is_win32s) {
-	if (strlen(pmport) == 0) {	/* get default printer */
-	    char buf[256];
-	    char *p;
+    if (strlen(pmport) == 0) {	/* get default printer */
+	char buf[256];
+	char *p;
 
-	    /* WinNT stores default printer in registry and win.ini */
-	    /* Win95 stores default printer in win.ini */
-	    GetProfileString("windows", "device", "", buf, sizeof(buf));
-	    if ((p = strchr(buf, ',')) != NULL)
-		*p = '\0';
-	    return gp_printfile_win32(filename, buf);
-	} else if (is_spool(pmport)) {
-	    if (strlen(pmport) >= 8)
-		return gp_printfile_win32(filename, (char *)pmport + 8);
-	    else
-		return gp_printfile_win32(filename, (char *)NULL);
-	} else
-	    return gp_printfile_gs16spl(filename, pmport);
-    } else {
-	/* Win32s */
-	if (is_spool(pmport)) {
-	    if (strlen(pmport) >= 8) {
-		/* extract port name from win.ini */
-		char driverbuf[256];
-		char *output;
-
-		GetProfileString("Devices", pmport + 8, "", driverbuf, sizeof(driverbuf));
-		strtok(driverbuf, ",");
-		output = strtok(NULL, ",");
-		return gp_printfile_gs16spl(filename, output);
-	    } else
-		return gp_printfile_gs16spl(filename, (char *)NULL);
-	} else
-	    return gp_printfile_gs16spl(filename, pmport);
-    }
+	/* WinNT stores default printer in registry and win.ini */
+	/* Win95 stores default printer in win.ini */
+	GetProfileString("windows", "device", "", buf, sizeof(buf));
+	if ((p = strchr(buf, ',')) != NULL)
+	    *p = '\0';
+	return gp_printfile_win32(filename, buf);
+    } else if (is_spool(pmport)) {
+	if (strlen(pmport) >= 8)
+	    return gp_printfile_win32(filename, (char *)pmport + 8);
+	else
+	    return gp_printfile_win32(filename, (char *)NULL);
+    } else
+	return gs_error_undefinedfilename;
 }
 
 #define PRINT_BUF_SIZE 16384u
@@ -348,21 +319,6 @@ get_queues(void)
     return buffer;
 }
 
-
-char *
-get_ports(void)
-{
-    char *buffer;
-
-    if (!is_win32s)
-	return get_queues();
-
-    if ((buffer = malloc(PORT_BUF_SIZE)) == (char *)NULL)
-	return NULL;
-    GetProfileString("ports", NULL, "", buffer, PORT_BUF_SIZE);
-    return buffer;
-}
-
 /* return TRUE if queuename available */
 /* return FALSE if cancelled or error */
 /* if queue non-NULL, use as suggested queue */
@@ -377,6 +333,7 @@ get_queuename(char *portname, const char *queue)
     if (buffer == NULL)
 	return FALSE;
     if ((queue == (char *)NULL) || (strlen(queue) == 0)) {
+        /* PROMPTING FOR A QUEUE IS CURRENTLY BROKEN */
 	/* select a queue */
 	iport = DialogBoxParam(phInstance, "QueueDlgBox", (HWND) NULL, SpoolDlgProc, (LPARAM) buffer);
 	if (!iport) {
@@ -399,59 +356,6 @@ get_queuename(char *portname, const char *queue)
     return TRUE;
 }
 
-/* return TRUE if portname available */
-/* return FALSE if cancelled or error */
-/* if port non-NULL, use as suggested port */
-BOOL
-get_portname(char *portname, const char *port)
-{
-    char *buffer;
-    char *p;
-    int i, iport;
-    char filename[MAXSTR];
-
-    buffer = get_ports();
-    if (buffer == NULL)
-	return FALSE;
-    if ((port == (char *)NULL) || (strlen(port) == 0)) {
-	if (buffer == (char *)NULL)
-	    return FALSE;
-	/* select a port */
-	iport = DialogBoxParam(phInstance, "SpoolDlgBox", (HWND) NULL, SpoolDlgProc, (LPARAM) buffer);
-	if (!iport) {
-	    free(buffer);
-	    return FALSE;
-	}
-	p = buffer;
-	for (i = 1; i < iport && strlen(p) != 0; i++)
-	    p += lstrlen(p) + 1;
-	strcpy(portname, p);
-    } else
-	strcpy(portname, port);
-
-    if (strlen(portname) == 0)
-	return FALSE;
-    if (strcmp(portname, "FILE:") == 0) {
-	OPENFILENAME ofn;
-
-	filename[0] = '\0';
-	memset(&ofn, 0, sizeof(OPENFILENAME));
-	ofn.lStructSize = sizeof(OPENFILENAME);
-	ofn.hwndOwner = (HWND) NULL;
-	ofn.lpstrFile = filename;
-	ofn.nMaxFile = sizeof(filename);
-	ofn.Flags = OFN_PATHMUSTEXIST;
-	if (!GetSaveFileName(&ofn)) {
-	    free(buffer);
-	    return FALSE;
-	}
-	strcpy(portname, filename);
-    }
-    free(buffer);
-    return TRUE;
-}
-
-
 /* True Win32 method, using OpenPrinter, WritePrinter etc. */
 private int
 gp_printfile_win32(const char *filename, char *port)
@@ -529,48 +433,6 @@ gp_printfile_win32(const char *filename, char *port)
 }
 
 
-/* Start a 16-bit application gs16spl.exe to print a file */
-/* Intended for Win32s where 16-bit spooler functions are not available */
-/* and Win32 spooler functions are not implemented. */
-int
-gp_printfile_gs16spl(const char *filename, const char *port)
-{
-/* Get printer port list from win.ini */
-    char portname[MAXSTR];
-    HINSTANCE hinst;
-    char command[MAXSTR];
-    char *p;
-    HWND hwndspl;
-
-    if (!get_portname(portname, port))
-	return FALSE;
-
-    /* get path to EXE - same as DLL */
-    GetModuleFileName(phInstance, command, sizeof(command));
-    if ((p = strrchr(command, '\\')) != (char *)NULL)
-	p++;
-    else
-	p = command;
-    *p = '\0';
-    sprintf(command + strlen(command), "gs16spl.exe %s %s",
-	    portname, filename);
-
-    hinst = (HINSTANCE) WinExec(command, SW_SHOWNORMAL);
-    if (hinst < (HINSTANCE) HINSTANCE_ERROR) {
-	char buf[MAXSTR];
-
-	sprintf(buf, "Can't run: %s", command);
-	MessageBox((HWND) NULL, buf, szAppName, MB_OK | MB_ICONSTOP);
-	return FALSE;
-    }
-    hwndspl = FindWindow(NULL, "GS Win32s/Win16 spooler");
-
-    while (IsWindow(hwndspl)) {
-	gp_check_interrupts(NULL);
-    }
-
-    return 0;
-}
 
 
 /******************************************************************/
@@ -803,3 +665,81 @@ int gp_enumerate_fonts_next(void *enum_state, char **fontname, char **path)
 void gp_enumerate_fonts_free(void *enum_state)
 {
 }           
+
+/* --------- 64 bit file access ----------- */
+/* MSVC versions before 8 doen't provide big files.
+   MSVC 8 doesn't distinguish big and small files,
+   but provide special positioning functions
+   to access data behind 4GB.
+   Currently we support 64 bits file access with MSVC only.
+ */
+
+FILE *gp_fopen_64(const char *filename, const char *mode)
+{
+    return fopen(filename, mode);
+}
+
+FILE *gp_open_scratch_file_64(const char *prefix,
+			   char fname[gp_file_name_sizeof],
+			   const char *mode)
+{
+    return gp_open_scratch_file(prefix, fname, mode);
+}
+
+FILE *gp_open_printer_64(char fname[gp_file_name_sizeof], int binary_mode)
+{
+    /* Assuming gp_open_scratch_file_64 is same as gp_open_scratch_file -
+       see the body of gp_open_printer. */
+    return gp_open_printer(fname, binary_mode);
+}
+
+#if defined(_MSC_VER) && _MSC_VER < 1400
+    int64_t _ftelli64( FILE *);
+    int _fseeki64( FILE *, int64_t, int);
+#endif
+
+int64_t gp_ftell_64(FILE *strm)
+{
+#if !defined(_MSC_VER)
+    return ftell(strm);
+#elif _MSC_VER < 1200
+    return ftell(strm);
+#else
+    return (int64_t)_ftelli64(strm);
+#endif
+}
+
+int gp_fseek_64(FILE *strm, int64_t offset, int origin)
+{
+#if !defined(_MSC_VER)
+    return fseek(strm, offset, origin);
+#elif _MSC_VER < 1200
+    long offset1 = (long)offset;
+    
+    if (offset != offset1)
+	return -1;
+    return fseek(strm, offset1, origin);
+#else
+    return _fseeki64(strm, offset, origin);
+#endif
+}
+
+/* -------------------------  _snprintf -----------------------------*/
+
+/* Microsoft Visual C++ 2005  doesn't properly define snprintf,
+   which is defined in the C standard ISO/IEC 9899:1999 (E) */
+
+int snprintf(char *buffer, size_t count, const char *format, ...)
+{
+    if (count > 0) {
+	va_list args;
+	int n; 
+
+	va_start(args, format);
+	n = _vsnprintf(buffer, count, format, args);
+	buffer[count - 1] = 0;
+	va_end(args);
+	return n;
+    } else
+	return 0;
+}
