@@ -1,10 +1,11 @@
-/* Copyright (C) 1996-2004 artofcode LLC. All rights reserved.
+/* Copyright (C) 2001-2007 artofcode LLC.
+   All Rights Reserved.
   
   This file is part of GNU ghostscript
 
-  GNU ghostscript is free software; you can redistribute it and/or modify it under
-  the terms of the GNU General Public License as published by the Free Software
-  Foundation; either version 2, or (at your option) any later version.
+  GNU ghostscript is free software; you can redistribute it and/or
+  modify it under the terms of the version 2 of the GNU General Public
+  License as published by the Free Software Foundation.
 
   GNU ghostscript is distributed in the hope that it will be useful, but WITHOUT
   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
@@ -16,7 +17,7 @@
 
 */
 
-/* $Id: gscdef.c,v 1.7 2006/06/16 12:55:03 Arabidopsis Exp $ */
+/* $Id: gscdef.c,v 1.10 2007/08/01 14:26:01 jemarch Exp $ */
 /* Configuration scalars */
 
 #include "std.h"
@@ -36,7 +37,7 @@ CONFIG_CONST long gs_buildtime = GS_BUILDTIME;
 
 #ifndef GS_COPYRIGHT
 #  define GS_COPYRIGHT\
-	"Copyright (C) 2006 artofcode LLC, Benicia, CA.  All rights reserved."
+	"Copyright (C) 2007 artofcode LLC, Benicia, CA.  All rights reserved."
 #endif
 const char *CONFIG_CONST gs_copyright = GS_COPYRIGHT;
 
@@ -48,7 +49,7 @@ const char *CONFIG_CONST gs_productfamily = GS_PRODUCTFAMILY;
 
 #ifndef GS_PRODUCT
 #  define GS_PRODUCT\
-	GS_PRODUCTFAMILY
+	GS_PRODUCTFAMILY ""
 #endif
 const char *CONFIG_CONST gs_product = GS_PRODUCT;
 
@@ -58,6 +59,14 @@ gs_program_name(void)
     return gs_product;
 }
 
+/* GS_VERSION must be defined in the Makefile. */
+CONFIG_CONST long gs_version = GS_VERSION;
+
+long gs_version_number(void)
+{
+	return gs_version;
+}
+
 /* GS_REVISION must be defined in the makefile. */
 CONFIG_CONST long gs_revision = GS_REVISION;
 
