@@ -1,10 +1,11 @@
-/* Copyright (C) 1997, 1998 Aladdin Enterprises.  All rights reserved.
+/* Copyright (C) 2001-2006 artofcode LLC.
+   All Rights Reserved.
   
   This file is part of GNU ghostscript
 
-  GNU ghostscript is free software; you can redistribute it and/or modify it under
-  the terms of the GNU General Public License as published by the Free Software
-  Foundation; either version 2, or (at your option) any later version.
+  GNU ghostscript is free software; you can redistribute it and/or
+  modify it under the terms of the version 2 of the GNU General Public
+  License as published by the Free Software Foundation.
 
   GNU ghostscript is distributed in the hope that it will be useful, but WITHOUT
   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
@@ -16,14 +17,15 @@
 
 */
 
-/* $Id: gscpixel.h,v 1.5 2006/03/08 12:30:24 Arabidopsis Exp $ */
+/* $Id: gscpixel.h,v 1.7 2007/08/01 14:26:03 jemarch Exp $ */
 /* Interface to DevicePixel color space */
 /* Requires gscspace.h */
 
 #ifndef gscpixel_INCLUDED
 #  define gscpixel_INCLUDED
 
-/* Initialize a DevicePixel color space. */
-int gs_cspace_init_DevicePixel(gs_memory_t *mem, gs_color_space * pcs, int depth);
+/* Construct a new DevicePixel color space. */
+int gs_cspace_new_DevicePixel(gs_memory_t *mem, gs_color_space **ppcs,
+			      int depth);
 
 #endif /* gscpixel_INCLUDED */
