@@ -3,9 +3,9 @@
   
   This file is part of GNU ghostscript
 
-  GNU ghostscript is free software; you can redistribute it and/or modify it under
-  the terms of the GNU General Public License as published by the Free Software
-  Foundation; either version 2, or (at your option) any later version.
+  GNU ghostscript is free software; you can redistribute it and/or
+  modify it under the terms of the version 2 of the GNU General Public
+  License as published by the Free Software Foundation.
 
   GNU ghostscript is distributed in the hope that it will be useful, but WITHOUT
   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
@@ -17,7 +17,7 @@
 
 */
 
-/* $Id: gsfcmap.c,v 1.7 2006/06/16 12:55:04 Arabidopsis Exp $ */
+/* $Id: gsfcmap.c,v 1.9 2007/08/01 14:26:05 jemarch Exp $ */
 /* CMap character decoding */
 #include "memory_.h"
 #include "string_.h"
@@ -87,6 +87,7 @@ identity_next_range(gs_cmap_ranges_enum_t *penum)
 
 	memset(penum->range.first, 0, pcimap->num_bytes);
 	memset(penum->range.last, 0xff, pcimap->num_bytes);
+	penum->range.size = pcimap->num_bytes;
 	penum->index = 1;
 	return 0;
     }
@@ -366,6 +367,9 @@ gs_cmap_lookups_enum_setup(gs_cmap_lookups_enum_t *penum,
  * For a random CMap, compute whether it is identity.
  * It is not applicable to gs_cmap_ToUnicode_t due to
  * different sizes of domain keys and range values.
+ * Note we reject CMaps with Registry=Artifex
+ * to force embedding special instandard CMaps,
+ * which are not commonly in use yet.
  */
 bool
 gs_cmap_compute_identity(const gs_cmap_t *pcmap, int font_index_only)
@@ -374,6 +378,9 @@ gs_cmap_compute_identity(const gs_cmap_t *pcmap, int font_index_only)
     gs_cmap_lookups_enum_t lenum;
     int code;
 
+    if (!bytes_compare(pcmap->CIDSystemInfo->Registry.data, pcmap->CIDSystemInfo->Registry.size,
+		    (const byte *)"Artifex", 7))
+	return false;
     for (gs_cmap_lookups_enum_init(pcmap, which, &lenum);
 	 (code = gs_cmap_enum_next_lookup(&lenum)) == 0; ) {
 	if (font_index_only >= 0 && lenum.entry.font_index != font_index_only)
