@@ -1,10 +1,11 @@
-/* Copyright (C) 1997, 2000 Aladdin Enterprises.  All rights reserved.
+/* Copyright (C) 2001-2006 artofcode LLC.
+   All Rights Reserved.
   
   This file is part of GNU ghostscript
 
-  GNU ghostscript is free software; you can redistribute it and/or modify it under
-  the terms of the GNU General Public License as published by the Free Software
-  Foundation; either version 2, or (at your option) any later version.
+  GNU ghostscript is free software; you can redistribute it and/or
+  modify it under the terms of the version 2 of the GNU General Public
+  License as published by the Free Software Foundation.
 
   GNU ghostscript is distributed in the hope that it will be useful, but WITHOUT
   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
@@ -16,7 +17,7 @@
 
 */
 
-/* $Id: gsfunc3.c,v 1.7 2006/06/16 12:55:04 Arabidopsis Exp $ */
+/* $Id: gsfunc3.c,v 1.9 2007/08/01 14:26:06 jemarch Exp $ */
 /* Implementation of LL3 Functions */
 #include "math_.h"
 #include "memory_.h"
@@ -367,8 +368,10 @@ fn_1ItSg_is_monotonic(const gs_function_t * pfn_common,
 	    vv1 = b1; /* Ignore a small noise */
 	if (vv0 == vv1)
 	    return 1;
-	if (vv0 < b1 && vv1 > b1)
-	    return 0; /* Consider stitches as monotonity beraks. */
+	if (vv0 < b1 && vv1 > b1) {
+	    *mask = 1;
+	    return 0; /* Consider stitches as monotonity breaks. */
+	}
 	e0 = pfn->params.Encode[2 * i];
 	e1 = pfn->params.Encode[2 * i + 1];
 	esmall = (float)1e-6 * any_abs(e1 - e0);
