@@ -1,10 +1,11 @@
-/* Copyright (C) 2000, 2001 Aladdin Enterprises.  All rights reserved.
+/* Copyright (C) 2001-2006 artofcode LLC.
+   All Rights Reserved.
   
   This file is part of GNU ghostscript
 
-  GNU ghostscript is free software; you can redistribute it and/or modify it under
-  the terms of the GNU General Public License as published by the Free Software
-  Foundation; either version 2, or (at your option) any later version.
+  GNU ghostscript is free software; you can redistribute it and/or
+  modify it under the terms of the version 2 of the GNU General Public
+  License as published by the Free Software Foundation.
 
   GNU ghostscript is distributed in the hope that it will be useful, but WITHOUT
   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
@@ -16,7 +17,7 @@
 
 */
 
-/* $Id: gsfunc4.c,v 1.7 2006/06/16 12:55:05 Arabidopsis Exp $ */
+/* $Id: gsfunc4.c,v 1.9 2007/08/01 14:26:06 jemarch Exp $ */
 /* Implementation of FunctionType 4 (PostScript Calculator) Functions */
 #include "math_.h"
 #include "memory_.h"
@@ -43,7 +44,10 @@ typedef struct gs_function_PtCr_s {
 private_st_function_PtCr();
 
 /* Define the maximum stack depth. */
-#define MAX_VSTACK 100		/* per documentation */
+#define MAX_VSTACK 256		/* Max 100 is enough per PDF spec, but we use this
+				 * for DeviceN handling. Must be at least as large
+				 * as the number of components
+				 */
 
 /* Define the structure of values on the stack. */
 typedef enum {
@@ -111,6 +115,9 @@ fn_PtCr_evaluate(const gs_function_t *pfn_common, const float *in, float *out)
     calc_value_t *vstack = &vstack_buf[1];
     calc_value_t *vsp = vstack + pfn->params.m;
     const byte *p = pfn->params.ops.data;
+    int repeat_count[MAX_PSC_FUNCTION_NESTING];
+    int repeat_proc_size[MAX_PSC_FUNCTION_NESTING];
+    int repeat_nesting_level = -1;
     int i;
 
     /*
@@ -213,8 +220,9 @@ fn_PtCr_evaluate(const gs_function_t *pfn_common, const float *in, float *out)
 
 	OP1(PtCr_if, E, E),		/* if */
 	OP_NONE(PtCr_else),		/* else */
-	OP_NONE(PtCr_return)		/* return */
-
+	OP_NONE(PtCr_return),		/* return */
+	OP1(E, PtCr_repeat, E),		/* repeat */
+	OP_NONE(PtCr_repeat_end)	/* repeat_end */
     };
 
     vstack[-1].type = CVT_NONE;  /* for type dispatch in empty stack case */
@@ -519,10 +527,23 @@ fn_PtCr_evaluate(const gs_function_t *pfn_common, const float *in, float *out)
 	    }
 	    /* falls through */
 	case PtCr_else:
-	    p += 2 + (p[0] << 8) + p[1];
+	    p += 2 + (p[0] << 8) + p[1];	/* skip the past body */
 	    continue;
 	case PtCr_return:
 	    goto fin;
+	case PtCr_repeat:
+	    repeat_nesting_level++;
+	    repeat_count[repeat_nesting_level] = vsp->value.i;
+	    repeat_proc_size[repeat_nesting_level] = 1 + (p[0] << 8) + p[1];	/* body size */
+	    --vsp;		/* pop the counter */
+	    p += 3 + (p[0] <<8) + p[1];		    /* advance just past the repeat_end */
+	    /* falls through */
+	case PtCr_repeat_end:
+	    if ((repeat_count[repeat_nesting_level])-- <= 0)
+		repeat_nesting_level--;
+	    else
+		p -= repeat_proc_size[repeat_nesting_level];
+	    continue;
 	}
     }
  fin:
@@ -618,6 +639,9 @@ calc_put_ops(stream *s, const byte *ops, uint size)
 	    spputc(s, '}');
 	    return 1;
 	/*case PtCr_return:*/	/* not possible */
+	case PtCr_repeat:		/* We shouldn't encounter this, but just in case */
+	case PtCr_repeat_end:
+	    return_error(gs_error_rangecheck);
 	default: {		/* must be < PtCr_NUM_OPS */
 		static const char *const op_names[] = {
 		    /* Keep this consistent with opcodes in gsfunc4.h! */
@@ -827,9 +851,11 @@ gs_function_PtCr_init(gs_function_t ** ppfn,
 		p += sizeof(int); break;
 	    case PtCr_float:
 		p += sizeof(float); break;
+	    case PtCr_repeat:
 	    case PtCr_if:
 	    case PtCr_else:
 		p += 2;
+	    case PtCr_repeat_end:
 	    case PtCr_true:
 	    case PtCr_false:
 		break;
