@@ -1,10 +1,11 @@
-/* Copyright (C) 1989, 2000 Aladdin Enterprises.  All rights reserved.
+/* Copyright (C) 2001-2006 artofcode LLC.
+   All Rights Reserved.
   
   This file is part of GNU ghostscript
 
-  GNU ghostscript is free software; you can redistribute it and/or modify it under
-  the terms of the GNU General Public License as published by the Free Software
-  Foundation; either version 2, or (at your option) any later version.
+  GNU ghostscript is free software; you can redistribute it and/or
+  modify it under the terms of the version 2 of the GNU General Public
+  License as published by the Free Software Foundation.
 
   GNU ghostscript is distributed in the hope that it will be useful, but WITHOUT
   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
@@ -16,9 +17,19 @@
 
 */
 
-/* $Id: gsmisc.c,v 1.7 2006/06/16 12:55:05 Arabidopsis Exp $ */
+/* $Id: gsmisc.c,v 1.10 2007/08/01 14:26:09 jemarch Exp $ */
 /* Miscellaneous utilities for Ghostscript library */
 
+
+/****************************************************************************
+ * This global should be put into either gs_imager_state or gs_device,
+ * but the "plumbing" retrofit is more than we need for things that were
+ * previously compile time modifications. CPSI_mode will at least make these
+ * dynamic (but NOT thread safe)!!!
+ ****************************************************************************
+ */
+int CPSI_mode = 0; /* false */	/* default GS behavior is 'false' */
+
 /*
  * In order to capture the original definition of sqrt, which might be
  * either a procedure or a macro and might not have an ANSI-compliant
@@ -50,6 +61,7 @@ orig_sqrt(double x)
 #include "gconfigv.h"		/* for USE_ASM */
 #include "gxfarith.h"
 #include "gxfixed.h"
+#include "stdint_.h"
 
 /* ------ Redirected stdout and stderr  ------ */
 
@@ -134,7 +146,7 @@ dprintf_file_tail(const char *file)
     const char *tail = file + strlen(file);
 
     while (tail > file &&
-	   (isalnum(tail[-1]) || tail[-1] == '.' || tail[-1] == '_')
+	   (isalnum((unsigned char)tail[-1]) || tail[-1] == '.' || tail[-1] == '_')
 	)
 	--tail;
     return tail;
@@ -246,19 +258,19 @@ int gs_throw_imp(const char *func, const char *file, int line, int op, int code,
 
     /* throw */
     if (op == 0)
-	errprintf(NULL, "+ %s:%d: %s(): %s\n", file, line, func, msg);
+	errprintf("+ %s:%d: %s(): %s\n", file, line, func, msg);
 
     /* rethrow */
     if (op == 1)
-	errprintf(NULL, "| %s:%d: %s(): %s\n", file, line, func, msg);
+	errprintf("| %s:%d: %s(): %s\n", file, line, func, msg);
 
     /* catch */
     if (op == 2)
-	errprintf(NULL, "- %s:%d: %s(): %s\n", file, line, func, msg);
+	errprintf("- %s:%d: %s(): %s\n", file, line, func, msg);
 
     /* warn */
     if (op == 3)
-	errprintf(NULL, "  %s:%d: %s(): %s\n", file, line, func, msg);
+	errprintf("  %s:%d: %s(): %s\n", file, line, func, msg);
 
     return code;
 }
@@ -1280,3 +1292,116 @@ gs_atan2_degrees(double y, double x, double *pangle)
     }
     return 0;
 }
+
+/*
+ * Define a function for finding intersection of small bars.
+ * Coordinates must be so small that their cubes fit into 60 bits.
+ * This function doesn't check intersections at end of bars,
+ * so  the caller must care of them on necessity.
+ * Returns : *ry is the Y-coordinate of the intersection
+ * truncated to 'fixed'; *ey is 1 iff the precise Y coordinate of
+ * the intersection is greater than *ry (used by the shading algorithm).
+ */
+bool 
+gx_intersect_small_bars(fixed q0x, fixed q0y, fixed q1x, fixed q1y, fixed q2x, fixed q2y, 
+			fixed q3x, fixed q3y, fixed *ry, fixed *ey)
+{
+    fixed dx1 = q1x - q0x, dy1 = q1y - q0y;
+    fixed dx2 = q2x - q0x, dy2 = q2y - q0y;
+    fixed dx3 = q3x - q0x, dy3 = q3y - q0y;
+
+    int64_t vp2a, vp2b, vp3a, vp3b;
+    int s2, s3;
+
+    if (dx1 == 0 && dy1 == 0)
+	return false; /* Zero length bars are out of interest. */
+    if (dx2 == 0 && dy2 == 0)
+	return false; /* Contacting ends are out of interest. */
+    if (dx3 == 0 && dy3 == 0)
+	return false; /* Contacting ends are out of interest. */
+    if (dx2 == dx1 && dy2 == dy1)
+	return false; /* Contacting ends are out of interest. */
+    if (dx3 == dx1 && dy3 == dy1)
+	return false; /* Contacting ends are out of interest. */
+    if (dx2 == dx3 && dy2 == dy3)
+	return false; /* Zero length bars are out of interest. */
+    vp2a = (int64_t)dx1 * dy2;
+    vp2b = (int64_t)dy1 * dx2; 
+    /* vp2 = vp2a - vp2b; It can overflow int64_t, but we only need the sign. */
+    if (vp2a > vp2b)
+	s2 = 1;
+    else if (vp2a < vp2b)
+	s2 = -1;
+    else 
+	s2 = 0;
+    vp3a = (int64_t)dx1 * dy3;
+    vp3b = (int64_t)dy1 * dx3; 
+    /* vp3 = vp3a - vp3b; It can overflow int64_t, but we only need the sign. */
+    if (vp3a > vp3b)
+	s3 = 1;
+    else if (vp3a < vp3b)
+	s3 = -1;
+    else 
+	s3 = 0;
+    if (s2 == 0) {
+	if (s3 == 0)
+	    return false; /* Collinear bars - out of interest. */
+	if (0 <= dx2 && dx2 <= dx1 && 0 <= dy2 && dy2 <= dy1) {
+	    /* The start of the bar 2 is in the bar 1. */
+	    *ry = q2y;
+	    *ey = 0;
+	    return true;
+	}
+    } else if (s3 == 0) {
+	if (0 <= dx3 && dx3 <= dx1 && 0 <= dy3 && dy3 <= dy1) {
+	    /* The end of the bar 2 is in the bar 1. */
+	    *ry = q3y;
+	    *ey = 0;
+	    return true;
+	}
+    } else if (s2 * s3 < 0) {
+	/* The intersection definitely exists, so the determinant isn't zero.  */
+	fixed d23x = dx3 - dx2, d23y = dy3 - dy2;
+	int64_t det = (int64_t)dx1 * d23y - (int64_t)dy1 * d23x;
+	int64_t mul = (int64_t)dx2 * d23y - (int64_t)dy2 * d23x;
+	{
+	    /* Assuming small bars : cubes of coordinates must fit into int64_t.
+	       curve_samples must provide that.  */
+	    int64_t num = dy1 * mul, iiy;
+	    fixed iy;
+	    fixed pry, pey;
+
+	    {	/* Likely when called form wedge_trap_decompose or constant_color_quadrangle,
+		   we always have det > 0 && num >= 0, but we check here for a safety reason. */
+		if (det < 0)
+		    num = -num, det = -det;
+		iiy = (num >= 0 ? num / det : (num - det + 1) / det);
+		iy = (fixed)iiy;
+		if (iy != iiy) {
+		    /* If it is inside the bars, it must fit into fixed. */
+		    return false;
+		}
+	    }
+	    if (dy1 > 0) {
+		if (iy < 0 || iy >= dy1)
+		    return false; /* Outside the bar 1. */
+	    } else {
+		if (iy > 0 || iy <= dy1)
+		    return false; /* Outside the bar 1. */
+	    }
+	    if (dy2 < dy3) {
+		if (iy <= dy2 || iy >= dy3)
+		    return false; /* Outside the bar 2. */
+	    } else {
+		if (iy >= dy2 || iy <= dy3)
+		    return false; /* Outside the bar 2. */
+	    }
+	    pry = q0y + (fixed)iy;
+	    pey = (iy * det < num ? 1 : 0);
+	    *ry = pry;
+	    *ey = pey;
+	}
+	return true;
+    }
+    return false;
+}
