@@ -1,10 +1,11 @@
-/* Copyright (C) 1999 Aladdin Enterprises.  All rights reserved.
+/* Copyright (C) 2001-2006 artofcode LLC.
+   All Rights Reserved.
   
   This file is part of GNU ghostscript
 
-  GNU ghostscript is free software; you can redistribute it and/or modify it under
-  the terms of the GNU General Public License as published by the Free Software
-  Foundation; either version 2, or (at your option) any later version.
+  GNU ghostscript is free software; you can redistribute it and/or
+  modify it under the terms of the version 2 of the GNU General Public
+  License as published by the Free Software Foundation.
 
   GNU ghostscript is distributed in the hope that it will be useful, but WITHOUT
   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
@@ -14,10 +15,9 @@
   ghostscript; see the file COPYING. If not, write to the Free Software Foundation,
   Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 
-  
 */
 
-/* $Id: gsptype1.c,v 1.7 2006/06/16 12:55:03 Arabidopsis Exp $ */
+/* $Id: gsptype1.c,v 1.9 2007/08/01 14:26:12 jemarch Exp $ */
 /* PatternType 1 pattern implementation */
 #include "math_.h"
 #include "gx.h"
@@ -89,23 +89,21 @@ private const gs_pattern_type_t gs_pattern1_type = {
  */
 int
 gs_cspace_build_Pattern1(gs_color_space ** ppcspace,
-		    const gs_color_space * pbase_cspace, gs_memory_t * pmem)
+		    gs_color_space * pbase_cspace, gs_memory_t * pmem)
 {
     gs_color_space *pcspace = 0;
-    int code;
 
     if (pbase_cspace != 0) {
 	if (gs_color_space_num_components(pcspace) < 0)		/* Pattern space */
 	    return_error(gs_error_rangecheck);
     }
-    code = gs_cspace_alloc(&pcspace, &gs_color_space_type_Pattern, pmem);
-    if (code < 0)
-	return code;
+    pcspace = gs_cspace_alloc(pmem, &gs_color_space_type_Pattern);
+    if (pcspace == NULL)
+	return_error(gs_error_VMerror);
     if (pbase_cspace != 0) {
 	pcspace->params.pattern.has_base_space = true;
-	gs_cspace_init_from((gs_color_space *) & (pcspace->params.pattern.base_space),
-			    pbase_cspace
-	    );
+	/* reference to base space shifts from pgs to pcs with no net change */
+	pcspace->base_space = pbase_cspace;
     } else
 	pcspace->params.pattern.has_base_space = false;
     *ppcspace = pcspace;
@@ -410,7 +408,7 @@ compute_inst_matrix(gs_pattern1_instance_t * pinst, const gs_state * saved,
     int code;
 
     /* Adjust the stepping matrix so all coefficients are >= 0. */
-    if (xx == 0 || yy == 0) {	/* We know that both xy and yx are non-zero. */
+    if (xx == 0 || yy == 0) { /* We know that both xy and yx are non-zero. */
 	double temp;
 
 	temp = xx, xx = yx, yx = temp;
@@ -505,7 +503,7 @@ gs_pattern1_set_color(const gs_client_color * pcc, gs_state * pgs)
     if (ptmplt->PaintType == 2) {
         const gs_color_space *  pcs = pgs->color_space;
 
-        pcs = (const gs_color_space *)&(pcs->params.pattern.base_space);
+        pcs = pcs->base_space;
         return pcs->type->set_overprint(pcs, pgs);
     } else {
         gs_overprint_params_t   params;
@@ -552,7 +550,7 @@ public_st_gx_strip_bitmap();
  */
 typedef struct pixmap_info_s {
     gs_depth_bitmap bitmap;	/* must be first */
-    const gs_color_space *pcspace;
+    gs_color_space *pcspace;
     uint white_index;
     void (*free_proc)(gs_memory_t *, void *, client_name_t);
 } pixmap_info;
@@ -620,8 +618,7 @@ image_PaintProc(const gs_client_color * pcolor, gs_state * pgs)
     const gs_depth_bitmap *pbitmap = &(ppmap->bitmap);
     gs_image_enum *pen =
         gs_image_enum_alloc(gs_state_memory(pgs), "image_PaintProc");
-    gs_color_space cs;
-    const gs_color_space *pcspace;
+    gs_color_space *pcspace;
     gx_image_enum_common_t *pie;
     /*
      * If the image is transparent then we want to do image type4 processing.
@@ -646,8 +643,7 @@ image_PaintProc(const gs_client_color * pcolor, gs_state * pgs)
 	return_error(gs_error_VMerror);
 
     if (ppmap->pcspace == 0) {
-        gs_cspace_init_DeviceGray(pgs->memory, &cs);
-        pcspace = &cs;
+	pcspace = gs_cspace_new_DeviceGray(pgs->memory);
     } else
         pcspace = ppmap->pcspace;
     gs_gsave(pgs);
@@ -700,7 +696,7 @@ bitmap_paint(gs_image_enum * pen, gs_data_image_t * pim,
     else
 	for (n = pim->Height; n > 0 && code >= 0; dp += raster, --n)
 	    code = gs_image_next(pen, dp, nbytes, &used);
-    code1 = gs_image_cleanup_and_free_enum(pen);
+    code1 = gs_image_cleanup_and_free_enum(pen, pgs);
     if (code >= 0 && code1 < 0)
 	code = code1;
     return code;
@@ -721,7 +717,7 @@ gs_makepixmappattern(
 			bool mask,
 			const gs_matrix * pmat,
 			long id,
-			const gs_color_space * pcspace,
+			gs_color_space * pcspace,
 			uint white_index,
 			gs_state * pgs,
 			gs_memory_t * mem
