@@ -3,9 +3,9 @@
   
   This file is part of GNU ghostscript
 
-  GNU ghostscript is free software; you can redistribute it and/or modify it under
-  the terms of the GNU General Public License as published by the Free Software
-  Foundation; either version 2, or (at your option) any later version.
+  GNU ghostscript is free software; you can redistribute it and/or
+  modify it under the terms of the version 2 of the GNU General Public
+  License as published by the Free Software Foundation.
 
   GNU ghostscript is distributed in the hope that it will be useful, but WITHOUT
   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
@@ -15,10 +15,9 @@
   ghostscript; see the file COPYING. If not, write to the Free Software Foundation,
   Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 
-  
 */
 
-/* $Id: gsshade.c,v 1.7 2006/06/16 12:55:04 Arabidopsis Exp $ */
+/* $Id: gsshade.c,v 1.9 2007/08/01 14:26:13 jemarch Exp $ */
 /* Constructors for shadings */
 #include "gx.h"
 #include "gscspace.h"
@@ -492,17 +491,22 @@ gs_shading_fill_path(const gs_shading_t *psh, /*const*/ gx_path *ppath,
 		    code = gs_note_error(gs_error_VMerror);
 		    goto out;
 		}
-		if ((code = gx_cpath_from_rectangle(path_clip, &path_box)) < 0 ||
-		    (code = shading_path_add_box(box_path, &psh->params.BBox,
-						pmat)) < 0 ||
-		    (code = gx_cpath_intersect(path_clip, box_path,
-					    gx_rule_winding_number, pis)) < 0
-		    )
-		    DO_NOTHING;
+		code = gx_cpath_from_rectangle(path_clip, &path_box);
+		if (code >= 0) {
+		    code = shading_path_add_box(box_path, &psh->params.BBox, pmat);
+		    if (code == gs_error_limitcheck) {
+			/* Ignore huge BBox - bug 689027. */
+			code = 0;
+		    } else if (code >= 0) {
+			code = gx_cpath_intersect(path_clip, box_path,
+					    gx_rule_winding_number, pis);
+			if (code >= 0)
+			    path_clip_set = true;
+		    }
+		}
 		gx_path_free(box_path, "shading_fill_path(box_path)");
 		if (code < 0)
 		    goto out;
-		path_clip_set = true;
 	    }
 	}
 	if (!path_clip_set) {
