@@ -3,9 +3,9 @@
   
   This file is part of GNU ghostscript
 
-  GNU ghostscript is free software; you can redistribute it and/or modify it under
-  the terms of the GNU General Public License as published by the Free Software
-  Foundation; either version 2, or (at your option) any later version.
+  GNU ghostscript is free software; you can redistribute it and/or
+  modify it under the terms of the version 2 of the GNU General Public
+  License as published by the Free Software Foundation.
 
   GNU ghostscript is distributed in the hope that it will be useful, but WITHOUT
   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
@@ -15,9 +15,8 @@
   ghostscript; see the file COPYING. If not, write to the Free Software Foundation,
   Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 
-  
 */
-/*$Id: gswts.c,v 1.5 2006/06/16 12:55:03 Arabidopsis Exp $ */
+/*$Id: gswts.c,v 1.7 2007/08/01 14:26:15 jemarch Exp $ */
 /* Screen generation for Well Tempered Screening. */
 #include "stdpre.h"
 #include <stdlib.h> /* for malloc */
@@ -122,9 +121,6 @@ typedef struct {
     int l;
 } wts_vec_t;
 
-private int
-gs_wts_to_buf(const wts_screen_t *ws, byte **pbuf);
-
 private void
 wts_vec_set(wts_vec_t *wv, int u, int v, int k, int l)
 {
