@@ -1,11 +1,11 @@
-/*
-  Copyright (C) 2001 artofcode LLC.
+/* Copyright (C) 2001-2006 artofcode LLC.
+   All Rights Reserved.
   
   This file is part of GNU ghostscript
 
-  GNU ghostscript is free software; you can redistribute it and/or modify it under
-  the terms of the GNU General Public License as published by the Free Software
-  Foundation; either version 2, or (at your option) any later version.
+  GNU ghostscript is free software; you can redistribute it and/or
+  modify it under the terms of the version 2 of the GNU General Public
+  License as published by the Free Software Foundation.
 
   GNU ghostscript is distributed in the hope that it will be useful, but WITHOUT
   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
@@ -15,11 +15,8 @@
   ghostscript; see the file COPYING. If not, write to the Free Software Foundation,
   Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 
-  
-
-  Author: Raph Levien <raph@artofcode.com>
 */
-/* $Id: gxblend.c,v 1.6 2006/03/08 12:30:25 Arabidopsis Exp $ */
+/* $Id: gxblend.c,v 1.8 2007/08/01 14:26:16 jemarch Exp $ */
 /* PDF 1.4 blending functions */
 
 #include "memory_.h"
