@@ -1,10 +1,11 @@
-/* Copyright (C) 1996, 1997, 1998, 1999 Aladdin Enterprises.  All rights reserved.
+/* Copyright (C) 2001-2006 artofcode LLC.
+   All Rights Reserved.
   
   This file is part of GNU ghostscript
 
-  GNU ghostscript is free software; you can redistribute it and/or modify it under
-  the terms of the GNU General Public License as published by the Free Software
-  Foundation; either version 2, or (at your option) any later version.
+  GNU ghostscript is free software; you can redistribute it and/or
+  modify it under the terms of the version 2 of the GNU General Public
+  License as published by the Free Software Foundation.
 
   GNU ghostscript is distributed in the hope that it will be useful, but WITHOUT
   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
@@ -14,10 +15,9 @@
   ghostscript; see the file COPYING. If not, write to the Free Software Foundation,
   Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 
-  
 */
 
-/*$Id: gxclimag.c,v 1.6 2006/06/16 12:55:04 Arabidopsis Exp $ */
+/*$Id: gxclimag.c,v 1.8 2007/08/01 14:26:17 jemarch Exp $ */
 /* Higher-level image operations for band lists */
 #include "math_.h"
 #include "memory_.h"
@@ -381,6 +381,9 @@ clist_begin_typed_image(gx_device * dev,
 	varying_depths ||
 	(code = gs_matrix_invert(&pim->ImageMatrix, &mat)) < 0 ||
 	(code = gs_matrix_multiply(&mat, &ctm_only(pis), &mat)) < 0 ||
+	/****** CAN'T HANDLE SOME TRANSFORMS ******/
+	(mat.tx != cdev->imager_state.ctm.tx) ||  
+	(mat.ty != cdev->imager_state.ctm.ty) ||  
 	!(cdev->disable_mask & clist_disable_nonrect_hl_image ?
 	  (is_xxyy(&mat) || is_xyyx(&mat)) :
 	  image_matrix_ok_to_band(&mat))
@@ -999,8 +1002,8 @@ cmd_put_color_mapping(gx_device_clist_writer * cldev,
     int code;
     const gx_device_halftone *pdht = pis->dev_ht;
 
-    /* Put out the halftone. */
-    if (pdht->id != cldev->device_halftone_id) {
+    /* Put out the halftone, if present. */
+    if (pdht && pdht->id != cldev->device_halftone_id) {
 	code = cmd_put_halftone(cldev, pdht);
 	if (code < 0)
 	    return code;
