@@ -1,10 +1,11 @@
-/* Copyright (C) 1997, 2000 Aladdin Enterprises.  All rights reserved.
+/* Copyright (C) 2001-2006 artofcode LLC.
+   All Rights Reserved.
   
   This file is part of GNU ghostscript
 
-  GNU ghostscript is free software; you can redistribute it and/or modify it under
-  the terms of the GNU General Public License as published by the Free Software
-  Foundation; either version 2, or (at your option) any later version.
+  GNU ghostscript is free software; you can redistribute it and/or
+  modify it under the terms of the version 2 of the GNU General Public
+  License as published by the Free Software Foundation.
 
   GNU ghostscript is distributed in the hope that it will be useful, but WITHOUT
   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
@@ -16,7 +17,7 @@
 
 */
 
-/* $Id: gxdevcli.h,v 1.7 2006/06/16 12:55:03 Arabidopsis Exp $ */
+/* $Id: gxdevcli.h,v 1.9 2007/08/01 14:26:21 jemarch Exp $ */
 /* Definitions for device clients */
 
 #ifndef gxdevcli_INCLUDED
@@ -700,7 +701,7 @@ typedef struct gx_device_cached_colors_s {
 	gx_device_cached_colors_t cached_colors;\
 	int width;			/* width in pixels */\
 	int height;			/* height in pixels */\
-        int TrayOrientation;            /* default 0 ( 90 180 270 ) if device supports */\
+        int LeadingEdge;                /* see below */\
 	float MediaSize[2];		/* media dimensions in points */\
 	float ImagingBBox[4];		/* imageable region in points */\
 	  bool ImagingBBox_set;\
@@ -722,6 +723,23 @@ typedef struct gx_device_cached_colors_s {
 	gx_page_device_procs page_procs;	/* must be last */\
 		/* end of std_device_body */\
 	gx_device_procs procs	/* object procedures */
+
+#define LEADINGEDGE_MASK 3
+#define LEADINGEDGE_SET_MASK (1 << 2)
+#define LEADINGEDGE_REQ_BIT (1 << 3)
+#define LEADINGEDGE_REQ_VAL_SHIFT 4
+#define LEADINGEDGE_REQ_VAL (LEADINGEDGE_MASK << LEADINGEDGE_REQ_VAL_SHIFT)
+/*
+ * The lower two bits of LeadingEdge correspond to the pagedevice
+ * parameter of the same name. The next bit (hex value 4) is set if
+ * the LeadingEdge was set explicitly by the user using setpagedevice.
+ * Otherwise, the value is the default as determined by the device (or
+ * zero if the device has no logic for setting LeadingEdge based on
+ * other parameters (such as %MediaSource). This field replaces the
+ * earlier TrayOrientation, which had a similar purpose but was not
+ * compatible with the PostScript spec.
+ */
+
 /*
  * Note: x/y_pixels_per_inch are here only for backward compatibility.
  * They should not be used in new code.
@@ -759,7 +777,6 @@ typedef struct gx_device_cached_colors_s {
 #define assign_dev_procs(todev, fromdev)\
   ((todev)->procs = (fromdev)->procs)
 
-
 /* ---------------- Device procedures ---------------- */
 
 /* Define an opaque type for parameter lists. */
@@ -1220,7 +1237,8 @@ typedef enum {
     pattern_manage__start_accum,
     pattern_manage__finish_accum,
     pattern_manage__load,
-    pattern_manage__shading_area
+    pattern_manage__shading_area,
+    pattern_manage__is_cpath_accum
 } pattern_manage_t;
 
 #define dev_t_proc_pattern_manage(proc, dev_t)\
