@@ -1,10 +1,11 @@
-/* Copyright (C) 1997, 1998, 1999, 2000 Aladdin Enterprises.  All rights reserved.
+/* Copyright (C) 2001-2006 artofcode LLC.
+   All Rights Reserved.
   
   This file is part of GNU ghostscript
 
-  GNU ghostscript is free software; you can redistribute it and/or modify it under
-  the terms of the GNU General Public License as published by the Free Software
-  Foundation; either version 2, or (at your option) any later version.
+  GNU ghostscript is free software; you can redistribute it and/or
+  modify it under the terms of the version 2 of the GNU General Public
+  License as published by the Free Software Foundation.
 
   GNU ghostscript is distributed in the hope that it will be useful, but WITHOUT
   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
@@ -14,10 +15,9 @@
   ghostscript; see the file COPYING. If not, write to the Free Software Foundation,
   Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 
-  
 */
 
-/* $Id: gxdevice.h,v 1.7 2006/06/16 12:55:04 Arabidopsis Exp $ */
+/* $Id: gxdevice.h,v 1.9 2007/08/01 14:26:21 jemarch Exp $ */
 /* Definitions for device implementors */
 
 #ifndef gxdevice_INCLUDED
@@ -583,4 +583,6 @@ int gdev_write_output_media(int index, gs_param_dict * pdict,
 
 int gdev_end_output_media(gs_param_list * mlist, gs_param_dict * pdict);
 
+void gx_device_request_leadingedge(gx_device *dev, int le_req);
+
 #endif /* gxdevice_INCLUDED */
