@@ -1,10 +1,11 @@
-/* Copyright (C) 1995, 2000 Aladdin Enterprises.  All rights reserved.
+/* Copyright (C) 2001-2006 artofcode LLC.
+   All Rights Reserved.
   
   This file is part of GNU ghostscript
 
-  GNU ghostscript is free software; you can redistribute it and/or modify it under
-  the terms of the GNU General Public License as published by the Free Software
-  Foundation; either version 2, or (at your option) any later version.
+  GNU ghostscript is free software; you can redistribute it and/or
+  modify it under the terms of the version 2 of the GNU General Public
+  License as published by the Free Software Foundation.
 
   GNU ghostscript is distributed in the hope that it will be useful, but WITHOUT
   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
@@ -14,10 +15,9 @@
   ghostscript; see the file COPYING. If not, write to the Free Software Foundation,
   Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 
-  
 */
 
-/* $Id: gxidata.c,v 1.5 2006/06/16 12:55:03 Arabidopsis Exp $ */
+/* $Id: gxidata.c,v 1.7 2007/08/01 14:26:25 jemarch Exp $ */
 /* Generic image enumeration and cleanup */
 #include "gx.h"
 #include "memory_.h"
@@ -434,8 +434,17 @@ gx_image1_end_image(gx_image_enum_common_t * info, bool draw_last)
 	if (code < 0)
 	    return code;
     }
+
+   /* release the reference to the target */
+    if ( penum->rop_dev )
+        gx_device_set_target((gx_device_forward *)penum->rop_dev, NULL);
+    if ( penum->clip_dev )
+        gx_device_set_target((gx_device_forward *)penum->clip_dev, NULL);
+    /* it is not clear (to me) why these are freed explicitly instead
+       of using reference counting */
     gs_free_object(mem, penum->rop_dev, "image RasterOp");
     gs_free_object(mem, penum->clip_dev, "image clipper");
+
     if (scaler != 0) {
 	(*scaler->template->release) ((stream_state *) scaler);
 	gs_free_object(mem, scaler, "image scaler state");
