@@ -1,10 +1,11 @@
-/* Copyright (C) 2000 Aladdin Enterprises.  All rights reserved.
+/* Copyright (C) 2001-2006 artofcode LLC.
+   All Rights Reserved.
   
   This file is part of GNU ghostscript
 
-  GNU ghostscript is free software; you can redistribute it and/or modify it under
-  the terms of the GNU General Public License as published by the Free Software
-  Foundation; either version 2, or (at your option) any later version.
+  GNU ghostscript is free software; you can redistribute it and/or
+  modify it under the terms of the version 2 of the GNU General Public
+  License as published by the Free Software Foundation.
 
   GNU ghostscript is distributed in the hope that it will be useful, but WITHOUT
   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
@@ -14,10 +15,9 @@
   ghostscript; see the file COPYING. If not, write to the Free Software Foundation,
   Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 
-  
 */
 
-/* $Id: gximag3x.c,v 1.7 2006/06/16 12:55:03 Arabidopsis Exp $ */
+/* $Id: gximag3x.c,v 1.9 2007/08/01 14:26:25 jemarch Exp $ */
 /* ImageType 3x image implementation */
 /****** THE REAL WORK IS NYI ******/
 #include "math_.h"		/* for ceil, floor */
@@ -67,7 +67,7 @@ gs_image3x_mask_init(gs_image3x_mask_t *pimm)
     pimm->MaskDict.BitsPerComponent = 0;	/* not supplied */
 }
 void
-gs_image3x_t_init(gs_image3x_t * pim, const gs_color_space * color_space)
+gs_image3x_t_init(gs_image3x_t * pim, gs_color_space * color_space)
 {
     gs_pixel_image_t_init((gs_pixel_image_t *) pim, color_space);
     pim->type = &gs_image_type_3x;
@@ -219,11 +219,9 @@ gx_begin_image3x_generic(gx_device * dev,
 	    minfo[i] = 0;
 	    continue;
 	}
-	pmcs =  gs_alloc_struct(mem, gs_color_space, &st_color_space,
-				"gx_begin_image3x_generic");
-	if (pmcs == 0)
-	    return_error(gs_error_VMerror);
-	gs_cspace_init_DevicePixel(mem, pmcs, penum->mask[i].depth);
+	code = gs_cspace_new_DevicePixel(mem, &pmcs, penum->mask[i].depth);
+	if (code < 0)
+	    return code;
 	mrect.p.x = mrect.p.y = 0;
 	mrect.q.x = penum->mask[i].width;
 	mrect.q.y = penum->mask[i].height;
