@@ -1,10 +1,11 @@
-/* Copyright (C) 1993, 1996, 1997, 1998, 1999 Aladdin Enterprises.  All rights reserved.
+/* Copyright (C) 2001-2006 artofcode LLC.
+   All Rights Reserved.
   
   This file is part of GNU ghostscript
 
-  GNU ghostscript is free software; you can redistribute it and/or modify it under
-  the terms of the GNU General Public License as published by the Free Software
-  Foundation; either version 2, or (at your option) any later version.
+  GNU ghostscript is free software; you can redistribute it and/or
+  modify it under the terms of the version 2 of the GNU General Public
+  License as published by the Free Software Foundation.
 
   GNU ghostscript is distributed in the hope that it will be useful, but WITHOUT
   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
@@ -14,10 +15,9 @@
   ghostscript; see the file COPYING. If not, write to the Free Software Foundation,
   Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 
-  
 */
 
-/* $Id: igc.c,v 1.7 2006/06/16 12:55:03 Arabidopsis Exp $ */
+/* $Id: igc.c,v 1.9 2007/08/01 14:26:35 jemarch Exp $ */
 /* Garbage collector for Ghostscript */
 #include "memory_.h"
 #include "ghost.h"
@@ -379,7 +379,6 @@ gs_gc_reclaim(vm_spaces * pspaces, bool global)
 	end_phase("finish trace");
     }
 
-#if NO_INVISIBLE_LEVELS
     /* Filter save change lists with removing elements,
        which point to unmarked blocks of refs. */
     {
@@ -391,7 +390,6 @@ gs_gc_reclaim(vm_spaces * pspaces, bool global)
 	    alloc_save__filter_changes(mem);
 	}
     }
-#endif
     /* Clear marks and relocation in spaces that are only being traced. */
     /* We have to clear the marks first, because we want the */
     /* relocation to wind up as o_untraced, not o_unmarked. */
