@@ -1,10 +1,11 @@
-/* Copyright (C) 1996-2003 artofcode LLC.  All rights reserved.
+/* Copyright (C) 2001-2006 artofcode LLC.
+   All Rights Reserved.
   
   This file is part of GNU ghostscript
 
-  GNU ghostscript is free software; you can redistribute it and/or modify it under
-  the terms of the GNU General Public License as published by the Free Software
-  Foundation; either version 2, or (at your option) any later version.
+  GNU ghostscript is free software; you can redistribute it and/or
+  modify it under the terms of the version 2 of the GNU General Public
+  License as published by the Free Software Foundation.
 
   GNU ghostscript is distributed in the hope that it will be useful, but WITHOUT
   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
@@ -14,10 +15,9 @@
   ghostscript; see the file COPYING. If not, write to the Free Software Foundation,
   Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 
-  
 */
 
-/* $Id: imainarg.c,v 1.8 2006/06/16 12:55:04 Arabidopsis Exp $ */
+/* $Id: imainarg.c,v 1.11 2007/08/01 14:26:36 jemarch Exp $ */
 /* Command line parsing and dispatching */
 #include "ctype_.h"
 #include "memory_.h"
@@ -356,7 +356,7 @@ run_stdin:
 		    char *sarg;
 
 		    if (arg[0] == '@' ||
-			(arg[0] == '-' && !isdigit(arg[1]))
+			(arg[0] == '-' && !isdigit((unsigned char)arg[1]))
 			)
 			break;
 		    sarg = arg_copy(arg, minst->heap);
@@ -588,9 +588,8 @@ run_stdin:
 			s_init(&astream, NULL);
 			sread_string(&astream,
 				     (const byte *)eqp, strlen(eqp));
-			scanner_state_init(&state, false);
-			code = scan_token(minst->i_ctx_p, &astream, &value,
-					  &state);
+			scanner_init_stream(&state, &astream);
+			code = scan_token(minst->i_ctx_p, &value, &state);
 			if (code) {
 			    puts(minst->heap, "-dname= must be followed by a valid token");
 			    return e_Fatal;
@@ -890,7 +889,7 @@ private const char help_usage2[] = "\
                                          embed %d or %ld for page #\n";
 private const char help_trailer[] = "\
 For more information, see %s.\n\
-Report bugs to %s.\n";
+Report bugs to %s, using the form in Bug-form.htm.\n";
 private const char help_devices[] = "Available devices:";
 private const char help_default_device[] = "Default output device:";
 private const char help_emulators[] = "Input formats:";
@@ -927,7 +926,10 @@ print_revision(const gs_main_instance *minst)
 private void
 print_version(const gs_main_instance *minst)
 {
-    printf_program_ident(minst->heap, NULL, gs_revision);
+	outprintf(minst->heap, "%d.%02d.%d",
+		(int)(gs_version / 10000),
+		(int)(gs_version / 100 % 100),
+		(int)(gs_version % 100));
 }
 
 /* Print usage information. */
