@@ -1,10 +1,11 @@
-/* Copyright (C) 1999 Aladdin Enterprises.  All rights reserved.
+/* Copyright (C) 2001-2006 artofcode LLC.
+   All Rights Reserved.
   
   This file is part of GNU ghostscript
 
-  GNU ghostscript is free software; you can redistribute it and/or modify it under
-  the terms of the GNU General Public License as published by the Free Software
-  Foundation; either version 2, or (at your option) any later version.
+  GNU ghostscript is free software; you can redistribute it and/or
+  modify it under the terms of the version 2 of the GNU General Public
+  License as published by the Free Software Foundation.
 
   GNU ghostscript is distributed in the hope that it will be useful, but WITHOUT
   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
@@ -14,10 +15,9 @@
   ghostscript; see the file COPYING. If not, write to the Free Software Foundation,
   Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 
-  
 */
 
-/* $Id: inobtokn.c,v 1.5 2005/12/13 16:57:25 jemarch Exp $ */
+/* $Id: inobtokn.c,v 1.7 2007/08/01 14:26:37 jemarch Exp $ */
 /* Dummy scan_binary_token for Level 1 systems */
 #include "ghost.h"
 #include "ierrors.h"
@@ -26,8 +26,7 @@
 #include "iscanbin.h"
 
 int
-scan_binary_token(i_ctx_t *i_ctx_p, stream *s, ref *pref,
-		  scanner_state *pstate)
+scan_binary_token(i_ctx_t *i_ctx_p, ref *pref, scanner_state *pstate)
 {
     return_error(e_unregistered);
 }
