@@ -1,10 +1,11 @@
-/* Copyright (C) 1989, 2000 Aladdin Enterprises.  All rights reserved.
+/* Copyright (C) 2001-2006 artofcode LLC.
+   All Rights Reserved.
   
   This file is part of GNU ghostscript
 
-  GNU ghostscript is free software; you can redistribute it and/or modify it under
-  the terms of the GNU General Public License as published by the Free Software
-  Foundation; either version 2, or (at your option) any later version.
+  GNU ghostscript is free software; you can redistribute it and/or
+  modify it under the terms of the version 2 of the GNU General Public
+  License as published by the Free Software Foundation.
 
   GNU ghostscript is distributed in the hope that it will be useful, but WITHOUT
   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
@@ -14,10 +15,9 @@
   ghostscript; see the file COPYING. If not, write to the Free Software Foundation,
   Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 
-  
 */
 
-/* $Id: iscanbin.c,v 1.6 2006/03/08 12:30:24 Arabidopsis Exp $ */
+/* $Id: iscanbin.c,v 1.8 2007/08/01 14:26:38 jemarch Exp $ */
 /* Ghostscript binary token scanner and writer */
 #include "math_.h"
 #include "memory_.h"
@@ -48,6 +48,7 @@ typedef enum {
     BT_SEQ_IEEE_LSB = 129,	/* IEEE float, little-endian */
     BT_SEQ_NATIVE_MSB = 130,	/* native floats, big-endian */
     BT_SEQ_NATIVE_LSB = 131,	/* native floats, little-endian */
+#define BT_IS_SEQ(btype) (((btype) & ~3) == BT_SEQ)
     BT_INT32_MSB = 132,
     BT_INT32_LSB = 133,
     BT_INT16_MSB = 134,
@@ -138,96 +139,169 @@ typedef enum {
 #define SIZEOF_BIN_SEQ_OBJ ((uint)8)
 
 /* Forward references */
-private int scan_bin_get_name(const gs_memory_t *mem, const ref *, int, ref *);
-private int scan_bin_num_array_continue(i_ctx_t *, stream *, ref *, scanner_state *);
-private int scan_bin_string_continue(i_ctx_t *, stream *, ref *, scanner_state *);
-private int scan_bos_continue(i_ctx_t *, stream *, ref *, scanner_state *);
+private int scan_bos(i_ctx_t *, ref *, scanner_state *);
+private void scan_bos_error(scanner_state *, const char *);
+private int scan_bin_scalar(i_ctx_t *, ref *, scanner_state *);
+private int scan_bin_get_name(scanner_state *, const gs_memory_t *mem, const ref *, int, ref *, const char *);
+private int scan_bin_num_array_continue(i_ctx_t *, ref *, scanner_state *);
+private int scan_bin_string_continue(i_ctx_t *, ref *, scanner_state *);
+private int scan_bos_continue(i_ctx_t *, ref *, scanner_state *);
 private byte *scan_bos_resize(i_ctx_t *, scanner_state *, uint, uint);
-private int scan_bos_string_continue(i_ctx_t *, stream *, ref *, scanner_state *);
+private int scan_bos_string_continue(i_ctx_t *, ref *, scanner_state *);
 
 /* Scan a binary token.  Called from the main scanner */
 /* when it encounters an ASCII code 128-159, */
 /* if binary tokens are being recognized (object format != 0). */
 int
-scan_binary_token(i_ctx_t *i_ctx_p, stream *s, ref *pref,
-		  scanner_state *pstate)
+scan_binary_token(i_ctx_t *i_ctx_p, ref *pref, scanner_state *pstate)
 {
+    stream *const s = pstate->s_file.value.pfile;
     scan_binary_state *const pbs = &pstate->s_ss.binary;
-
     s_declare_inline(s, p, rlimit);
-    int num_format, code;
-    uint arg;
+    int btype, code;
     uint wanted;
-    uint rcnt;
+
+    s_begin_inline(s, p, rlimit);
+    pbs->token_type = btype = *p;
+    wanted = bin_token_bytes[btype - MIN_BIN_TOKEN_TYPE] - 1;
+    if (rlimit - p < wanted) {
+	s_end_inline(s, p - 1, rlimit);
+	pstate->s_scan_type = scanning_none;
+	code = scan_Refill;
+    } else {
+	pbs->num_format = bin_token_num_formats[btype - MIN_BIN_TOKEN_TYPE];
+	if (BT_IS_SEQ(btype))
+	    code = scan_bos(i_ctx_p, pref, pstate);
+	else
+	    code = scan_bin_scalar(i_ctx_p, pref, pstate);
+    }
+    if (code == scan_Refill && s->end_status == EOFC)
+	code = gs_note_error(e_syntaxerror);
+    if (code < 0 && pstate->s_error.string[0] == 0)
+	snprintf(pstate->s_error.string, sizeof(pstate->s_error.string),
+		 "binary token, type=%d", btype);
+    return code;
+}
+
+/* Scan a binary object sequence. */
+private int
+scan_bos(i_ctx_t *i_ctx_p, ref *pref, scanner_state *pstate)
+{
+    stream *const s = pstate->s_file.value.pfile;
+    scan_binary_state *const pbs = &pstate->s_ss.binary;
+    s_declare_inline(s, p, rlimit);
+    int num_format = pbs->num_format;
+    int code;
+
+    s_begin_inline(s, p, rlimit);
+    {
+	uint rcnt = rlimit - p;
+	uint top_size = p[1];
+	uint hsize, size;
+
+	if (top_size == 0) {
+	    /* Extended header (2-byte array size, 4-byte length) */
+	    ulong lsize;
+
+	    if (rcnt < 7) {
+		s_end_inline(s, p - 1, rlimit);
+		pstate->s_scan_type = scanning_none;
+		return scan_Refill;
+	    }
+	    pbs->top_size = top_size = sdecodeushort(p + 2, num_format);
+	    pbs->lsize = lsize = sdecodelong(p + 4, num_format);
+	    if ((size = lsize) != lsize) {
+		scan_bos_error(pstate, "bin obj seq length too large");
+		return_error(e_limitcheck);
+	    }
+	    hsize = 8;
+	} else {
+	    /* Normal header (1-byte array size, 2-byte length). */
+	    /* We already checked rcnt >= 3. */
+	    pbs->top_size = top_size;
+	    pbs->lsize = size = sdecodeushort(p + 2, num_format);
+	    hsize = 4;
+	}
+	if (size < hsize || (size - hsize) >> 3 < top_size) {
+	    scan_bos_error(pstate, "sequence too short");
+	    return_error(e_syntaxerror); /* size too small */
+	}
+        { /* Preliminary syntax check to avoid potentialy large
+           * memory allocation on junk data. Bug 688833
+           */
+          const unsigned char *q, *rend = p + hsize + top_size*8;
+
+          if (rend > rlimit)
+              rend = rlimit; 
+          for (q = p + hsize + 1; q < rend; q += 8) {
+             int c = q[-1] & 0x7f;
+             if (c > 10 && c != BS_TYPE_DICTIONARY) {
+		scan_bos_error(pstate, "invalid object type");
+		return_error(e_syntaxerror);
+             }
+             if (*q != 0) {
+		scan_bos_error(pstate, "non-zero unused field");
+		return_error(e_syntaxerror);
+             }
+          }
+        }
+	/*
+	 * Preallocate an array large enough for the worst case,
+	 * namely, all objects and no strings.  Note that we must
+	 * divide size by 8, not sizeof(ref), since array elements
+	 * in binary tokens always occupy 8 bytes regardless of the
+	 * size of a ref.
+	 */
+	code = ialloc_ref_array(&pbs->bin_array,
+				a_all + a_executable, size / 8,
+				"binary object sequence(objects)");
+	if (code < 0)
+	    return code;
+	p += hsize - 1;
+	size -= hsize;
+	s_end_inline(s, p, rlimit);
+	pbs->max_array_index = pbs->top_size = top_size;
+	pbs->min_string_index = pbs->size = size;
+	pbs->index = 0;
+	pstate->s_da.is_dynamic = false;
+	pstate->s_da.base = pstate->s_da.next =
+	    pstate->s_da.limit = pstate->s_da.buf;
+	code = scan_bos_continue(i_ctx_p, pref, pstate);
+	if (code == scan_Refill || code < 0) {
+	    /* Clean up array for GC. */
+	    uint index = pbs->index;
+
+	    refset_null(pbs->bin_array.value.refs + index,
+			r_size(&pbs->bin_array) - index);
+	}
+	return code;
+    }
+}
+
+/* Report an error in a binary object sequence. */
+private void
+scan_bos_error(scanner_state *pstate, const char *msg)
+{
+    snprintf(pstate->s_error.string, sizeof(pstate->s_error.string),
+	     "bin obj seq, type=%d, elements=%u, size=%lu, %s",
+	     pstate->s_ss.binary.token_type,
+	     pstate->s_ss.binary.top_size,
+	     pstate->s_ss.binary.lsize, msg);
+}
+
+/* Scan a non-sequence binary token. */
+private int
+scan_bin_scalar(i_ctx_t *i_ctx_p, ref *pref, scanner_state *pstate)
+{
+    stream *const s = pstate->s_file.value.pfile;
+    scan_binary_state *const pbs = &pstate->s_ss.binary;
+    s_declare_inline(s, p, rlimit);
+    int num_format = pbs->num_format, code;
+    uint wanted, arg;
 
     s_begin_inline(s, p, rlimit);
     wanted = bin_token_bytes[*p - MIN_BIN_TOKEN_TYPE] - 1;
-    rcnt = rlimit - p;
-    if (rcnt < wanted) {
-	s_end_inline(s, p - 1, rlimit);
-	pstate->s_scan_type = scanning_none;
-	return scan_Refill;
-    }
-    num_format = bin_token_num_formats[*p - MIN_BIN_TOKEN_TYPE];
     switch (*p) {
-	case BT_SEQ_IEEE_MSB:
-	case BT_SEQ_IEEE_LSB:
-	case BT_SEQ_NATIVE_MSB:
-	case BT_SEQ_NATIVE_LSB:{
-		uint top_size = p[1];
-		uint hsize, size;
-
-		pbs->num_format = num_format;
-		if (top_size == 0) {
-		    /* Extended header (2-byte array size, 4-byte length) */
-		    ulong lsize;
-
-		    if (rcnt < 7) {
-			s_end_inline(s, p - 1, rlimit);
-			pstate->s_scan_type = scanning_none;
-			return scan_Refill;
-		    }
-		    if (p[1] != 0) /* reserved, must be 0 */
-			return_error(e_syntaxerror);
-		    top_size = sdecodeushort(p + 2, num_format);
-		    lsize = sdecodelong(p + 4, num_format);
-		    if ((size = lsize) != lsize)
-			return_error(e_limitcheck);
-		    hsize = 8;
-		} else {
-		    /* Normal header (1-byte array size, 2-byte length). */
-		    /* We already checked rcnt >= 3. */
-		    size = sdecodeushort(p + 2, num_format);
-		    hsize = 4;
-		}
-		if (size < hsize)
-		    return_error(e_syntaxerror);
-		/* Preallocate an array large enough for the worst case, */
-		/* namely, all objects and no strings. */
-		code = ialloc_ref_array(&pbs->bin_array,
-				   a_all + a_executable, size / sizeof(ref),
-					"binary object sequence(objects)");
-		if (code < 0)
-		    return code;
-		p += hsize - 1;
-		size -= hsize;
-		s_end_inline(s, p, rlimit);
-		pbs->max_array_index = pbs->top_size = top_size;
-		pbs->min_string_index = pbs->size = size;
-		pbs->index = 0;
-		pstate->s_da.is_dynamic = false;
-		pstate->s_da.base = pstate->s_da.next =
-		    pstate->s_da.limit = pstate->s_da.buf;
-		code = scan_bos_continue(i_ctx_p, s, pref, pstate);
-		if (code == scan_Refill || code < 0) {
-		    /* Clean up array for GC. */
-		    uint index = pbs->index;
-
-		    refset_null(pbs->bin_array.value.refs + index,
-				r_size(&pbs->bin_array) - index);
-		}
-		return code;
-	    }
 	case BT_INT8:
 	    make_int(pref, (p[1] ^ 128) - 128);
 	    s_end_inline(s, p + 1, rlimit);
@@ -237,7 +311,7 @@ scan_binary_token(i_ctx_t *i_ctx_p, stream *s, ref *pref,
 	    if (!num_is_valid(num_format))
 		return_error(e_syntaxerror);
 	    wanted = 1 + encoded_number_bytes(num_format);
-	    if (rcnt < wanted) {
+	    if (rlimit - p < wanted) {
 		s_end_inline(s, p - 1, rlimit);
 		pstate->s_scan_type = scanning_none;
 		return scan_Refill;
@@ -299,7 +373,7 @@ scan_binary_token(i_ctx_t *i_ctx_p, stream *s, ref *pref,
 		s_end_inline(s, p, rlimit);
 		pstate->s_da.base = pstate->s_da.next = str;
 		pstate->s_da.limit = str + arg;
-		code = scan_bin_string_continue(i_ctx_p, s, pref, pstate);
+		code = scan_bin_string_continue(i_ctx_p, pref, pstate);
 		if (code == scan_Refill || code < 0) {
 		    pstate->s_da.is_dynamic = true;
 		    make_null(&pbs->bin_array);		/* clean up for GC */
@@ -308,27 +382,27 @@ scan_binary_token(i_ctx_t *i_ctx_p, stream *s, ref *pref,
 		return code;
 	    }
 	case BT_LITNAME_SYSTEM:
-	    code = scan_bin_get_name(imemory, system_names_p, p[1], pref);
+	    code = scan_bin_get_name(pstate, imemory, system_names_p, p[1],
+				     pref, "system");
 	    goto lname;
 	case BT_EXECNAME_SYSTEM:
-	    code = scan_bin_get_name(imemory, system_names_p, p[1], pref);
+	    code = scan_bin_get_name(pstate, imemory, system_names_p, p[1],
+				     pref, "system");
 	    goto xname;
 	case BT_LITNAME_USER:
-	    code = scan_bin_get_name(imemory, user_names_p, p[1], pref);
+	    code = scan_bin_get_name(pstate, imemory, user_names_p, p[1],
+				     pref, "user");
 	  lname:
 	    if (code < 0)
 		return code;
-	    if (!r_has_type(pref, t_name))
-		return_error(e_undefined);
 	    s_end_inline(s, p + 1, rlimit);
 	    return 0;
 	case BT_EXECNAME_USER:
-	    code = scan_bin_get_name(imemory, user_names_p, p[1], pref);
+	    code = scan_bin_get_name(pstate, imemory, user_names_p, p[1],
+				     pref, "user");
 	  xname:
 	    if (code < 0)
 		return code;
-	    if (!r_has_type(pref, t_name))
-		return_error(e_undefined);
 	    r_set_attrs(pref, a_executable);
 	    s_end_inline(s, p + 1, rlimit);
 	    return 0;
@@ -345,7 +419,7 @@ scan_binary_token(i_ctx_t *i_ctx_p, stream *s, ref *pref,
 	    pbs->index = 0;
 	    p += 3;
 	    s_end_inline(s, p, rlimit);
-	    code = scan_bin_num_array_continue(i_ctx_p, s, pref, pstate);
+	    code = scan_bin_num_array_continue(i_ctx_p, pref, pstate);
 	    if (code == scan_Refill || code < 0) {
 		/* Make sure the array is clean for the GC. */
 		refset_null(pbs->bin_array.value.refs + pbs->index,
@@ -359,18 +433,27 @@ scan_binary_token(i_ctx_t *i_ctx_p, stream *s, ref *pref,
 
 /* Get a system or user name. */
 private int
-scan_bin_get_name(const gs_memory_t *mem, const ref *pnames /*t_array*/, int index, ref *pref)
+scan_bin_get_name(scanner_state *pstate, const gs_memory_t *mem,
+		  const ref *pnames /*t_array*/, int index, ref *pref,
+		  const char *usstring)
 {
-    if (pnames == 0)
-	return_error(e_rangecheck);
-    return array_get(mem, pnames, (long)index, pref);
+    /* Convert all errors to e_undefined to match Adobe. */
+    if (pnames == 0 || array_get(mem, pnames, (long)index, pref) < 0 ||
+	!r_has_type(pref, t_name)) {
+	snprintf(pstate->s_error.string,
+		 sizeof(pstate->s_error.string),
+		 "%s%d", usstring, index);
+	pstate->s_error.is_name = true;
+	return_error(e_undefined);
+    }
+    return 0;
 }
 
 /* Continue collecting a binary string. */
 private int
-scan_bin_string_continue(i_ctx_t *i_ctx_p, stream * s, ref * pref,
-			 scanner_state * pstate)
+scan_bin_string_continue(i_ctx_t *i_ctx_p, ref * pref, scanner_state * pstate)
 {
+    stream *const s = pstate->s_file.value.pfile;
     byte *q = pstate->s_da.next;
     uint wanted = pstate->s_da.limit - q;
     uint rcnt;
@@ -394,9 +477,10 @@ scan_bin_string_continue(i_ctx_t *i_ctx_p, stream * s, ref * pref,
 
 /* Continue scanning a binary number array. */
 private int
-scan_bin_num_array_continue(i_ctx_t *i_ctx_p, stream * s, ref * pref,
+scan_bin_num_array_continue(i_ctx_t *i_ctx_p, ref * pref,
 			    scanner_state * pstate)
 {
+    stream *const s = pstate->s_file.value.pfile;
     scan_binary_state *const pbs = &pstate->s_ss.binary;
     uint index = pbs->index;
     ref *np = pbs->bin_array.value.refs + index;
@@ -418,6 +502,7 @@ scan_bin_num_array_continue(i_ctx_t *i_ctx_p, stream * s, ref * pref,
 		sbufskip(s, wanted);
 		break;
 	    case t_null:
+		scan_bos_error(pstate, "bad number format");
 		return_error(e_syntaxerror);
 	    default:
 		return code;
@@ -437,9 +522,9 @@ scan_bin_num_array_continue(i_ctx_t *i_ctx_p, stream * s, ref * pref,
  * all the pointers.
  */
 private int
-scan_bos_continue(i_ctx_t *i_ctx_p, register stream * s, ref * pref,
-		  scanner_state * pstate)
+scan_bos_continue(i_ctx_t *i_ctx_p, ref * pref, scanner_state * pstate)
 {
+    stream *const s = pstate->s_file.value.pfile;
     scan_binary_state *const pbs = &pstate->s_ss.binary;
     s_declare_inline(s, p, rlimit);
     uint max_array_index = pbs->max_array_index;
@@ -466,35 +551,60 @@ scan_bos_continue(i_ctx_t *i_ctx_p, register stream * s, ref * pref,
 	    pstate->s_scan_type = scanning_binary;
 	    return scan_Refill;
 	}
-	if (p[2] != 0) /* reserved, must be 0 */
+	if (p[2] != 0) { /* reserved, must be 0 */
+	    scan_bos_error(pstate, "non-zero unused field");
 	    return_error(e_syntaxerror);
+	}
 	attrs = (p[1] & 128 ? a_executable : 0);
+	/*
+	 * We always decode all 8 bytes of the object, so we can signal
+	 * syntaxerror if any unused field is non-zero (per PLRM).
+	 */
+  	osize = sdecodeushort(p + 3, num_format);
+	value = sdecodelong(p + 5, num_format);
 	switch (p[1] & 0x7f) {
 	    case BS_TYPE_NULL:
+		if (osize | value) { /* unused */
+		    scan_bos_error(pstate, "non-zero unused field");
+		    return_error(e_syntaxerror);
+		}
 		make_null(op);
 		break;
 	    case BS_TYPE_INTEGER:
-		make_int(op, sdecodelong(p + 5, num_format));
+		if (osize) {	/* unused */
+		    scan_bos_error(pstate, "non-zero unused field");
+		    return_error(e_syntaxerror);
+		}
+		make_int(op, value);
 		break;
 	    case BS_TYPE_REAL:{
 		    float vreal;
 
-		    osize = sdecodeushort(p + 3, num_format);
 		    if (osize != 0) {	/* fixed-point number */
-			value = sdecodelong(p + 5, num_format);
+			if (osize > 31) {
+			    scan_bos_error(pstate, "invalid number format");
+			    return_error(e_syntaxerror);
+			}
 			/* ldexp requires a signed 2nd argument.... */
 			vreal = (float)ldexp((double)value, -(int)osize);
 		    } else {
-			vreal = sdecodefloat(p + 5, num_format);
+			code = sdecode_float(p + 5, num_format, &vreal);
+			if (code < 0) {
+			    scan_bos_error(pstate, "invalid real number");
+			    return code;
+			}
 		    }
 		    make_real(op, vreal);
 		    break;
 		}
 	    case BS_TYPE_BOOLEAN:
-		make_bool(op, sdecodelong(p + 5, num_format) != 0);
+		if (osize) {	/* unused */
+		    scan_bos_error(pstate, "non-zero unused field");
+		    return_error(e_syntaxerror);
+		}
+		make_bool(op, value != 0);
 		break;
 	    case BS_TYPE_STRING:
-		osize = sdecodeushort(p + 3, num_format);
 		attrs |= a_all;
 	      str:
 		if (osize == 0) {
@@ -503,11 +613,12 @@ scan_bos_continue(i_ctx_t *i_ctx_p, register stream * s, ref * pref,
 		    make_empty_string(op, attrs);
 		    break;
 		}
-		value = sdecodelong(p + 5, num_format);
 		if (value < max_array_index * SIZEOF_BIN_SEQ_OBJ ||
 		    value + osize > size
-		    )
+		    ) {
+		    scan_bos_error(pstate, "invalid string offset");
 		    return_error(e_syntaxerror);
+		}
 		if (value < min_string_index) {
 		    /* We have to (re)allocate the strings. */
 		    uint str_size = size - value;
@@ -534,19 +645,19 @@ scan_bos_continue(i_ctx_t *i_ctx_p, register stream * s, ref * pref,
 		attrs |= a_readonly;	/* mark as executable for later */
 		/* falls through */
 	    case BS_TYPE_NAME:
-		osize = sdecodeushort(p + 3, num_format);
-		value = sdecodelong(p + 5, num_format);
 		switch (osize) {
 		    case 0:
-			code = array_get(imemory, user_names_p, value, op);
+			code = scan_bin_get_name(pstate, imemory,
+						 user_names_p, value, op,
+						 "user");
 			goto usn;
 		    case 0xffff:
-			code = array_get(imemory, system_names_p, value, op);
+			code = scan_bin_get_name(pstate, imemory,
+						 system_names_p, value, op,
+						 "system");
 		      usn:
 			if (code < 0)
 			    return code;
-			if (!r_has_type(op, t_name))
-			    return_error(e_undefined);
 			r_set_attrs(op, attrs);
 			break;
 		    default:
@@ -554,14 +665,14 @@ scan_bos_continue(i_ctx_t *i_ctx_p, register stream * s, ref * pref,
 		}
 		break;
 	    case BS_TYPE_ARRAY:
-		osize = sdecodeushort(p + 3, num_format);
 		atype = t_array;
 	      arr:
-		value = sdecodelong(p + 5, num_format);
 		if (value + osize > min_string_index ||
 		    value & (SIZEOF_BIN_SEQ_OBJ - 1)
-		    )
+		    ) {
+		    scan_bos_error(pstate, "bad array offset");
 		    return_error(e_syntaxerror);
+		}
 		{
 		    uint aindex = value / SIZEOF_BIN_SEQ_OBJ;
 
@@ -573,15 +684,19 @@ scan_bos_continue(i_ctx_t *i_ctx_p, register stream * s, ref * pref,
 		}
 		break;
 	    case BS_TYPE_DICTIONARY:	/* EXTENSION */
-		osize = sdecodeushort(p + 3, num_format);
 		if ((osize & 1) != 0 && osize != 1)
 		    return_error(e_syntaxerror);
 		atype = t_mixedarray;	/* mark as dictionary */
 		goto arr;
 	    case BS_TYPE_MARK:
+		if (osize | value) { /* unused */
+		    scan_bos_error(pstate, "non-zero unused field");
+		    return_error(e_syntaxerror);
+		}
 		make_mark(op);
 		break;
 	    default:
+		scan_bos_error(pstate, "invalid object type");
 		return_error(e_syntaxerror);
 	}
     }
@@ -591,7 +706,7 @@ scan_bos_continue(i_ctx_t *i_ctx_p, register stream * s, ref * pref,
     pbs->index = max_array_index;
     iresize_ref_array(&pbs->bin_array, max_array_index,
 		      "binary object sequence(objects)");
-    code = scan_bos_string_continue(i_ctx_p, s, pref, pstate);
+    code = scan_bos_string_continue(i_ctx_p, pref, pstate);
     if (code == scan_Refill)
 	pbs->cont = scan_bos_string_continue;
     return code;
@@ -626,13 +741,13 @@ scan_bos_resize(i_ctx_t *i_ctx_p, scanner_state * pstate, uint new_size,
 
 /* Continue reading the strings for a binary object sequence. */
 private int
-scan_bos_string_continue(i_ctx_t *i_ctx_p, register stream * s, ref * pref,
+scan_bos_string_continue(i_ctx_t *i_ctx_p, ref * pref,
 			 scanner_state * pstate)
 {
     scan_binary_state *const pbs = &pstate->s_ss.binary;
     ref rstr;
     ref *op;
-    int code = scan_bin_string_continue(i_ctx_p, s, &rstr, pstate);
+    int code = scan_bin_string_continue(i_ctx_p, &rstr, pstate);
     uint space = ialloc_space(idmemory);
     bool rescan = false;
     uint i;
@@ -651,8 +766,7 @@ scan_bos_string_continue(i_ctx_t *i_ctx_p, register stream * s, ref * pref,
 		    break;
 		/* This is actually a name; look it up now. */
 		{
-		    uint attrs =
-		    (r_has_attr(op, a_executable) ? a_executable : 0);
+		    uint attrs = r_type_attrs(op) & (a_read | a_executable);
 
 		    code = name_ref(imemory, op->value.bytes, r_size(op), op, 1);
 		    if (code < 0)
@@ -740,6 +854,11 @@ scan_bos_string_continue(i_ctx_t *i_ctx_p, register stream * s, ref * pref,
 
 /* ---------------- Writing ---------------- */
 
+/*
+ * Encode a single object for a binary object sequence, for printobject and
+ * write object.  Note that this does not modify the always-unused byte (1),
+ * but it always write bytes 0 and 2-7.
+ */
 int
 encode_binary_token(i_ctx_t *i_ctx_p, const ref *obj, long *ref_offset,
 		    long *char_offset, byte *str)
@@ -747,16 +866,16 @@ encode_binary_token(i_ctx_t *i_ctx_p, const ref *obj, long *ref_offset,
     bin_seq_type_t type;
     uint size = 0;
     int format = (int)ref_binary_object_format.value.intval;
-    long value;
+    long value = 0;
     ref nstr;
 
     switch (r_type(obj)) {
 	case t_null:
 	    type = BS_TYPE_NULL;
-	    goto tx;
+	    break;		/* always set all fields */
 	case t_mark:
 	    type = BS_TYPE_MARK;
-	    goto tx;
+	    break;		/* always set all fields */
 	case t_integer:
 	    type = BS_TYPE_INTEGER;
 	    value = obj->value.intval;
@@ -820,7 +939,6 @@ nos:
 	    str[4] = v0, str[5] = v1, str[6] = v2, str[7] = v3;
 	}
     }
-tx:
     if (r_has_attr(obj, a_executable))
 	type += BS_EXECUTABLE;
     str[0] = (byte) type;
