@@ -1,10 +1,11 @@
-/* Copyright (C) 1999 Aladdin Enterprises.  All rights reserved.
+/* Copyright (C) 2001-2006 artofcode LLC.
+   All Rights Reserved.
   
   This file is part of GNU ghostscript
 
-  GNU ghostscript is free software; you can redistribute it and/or modify it under
-  the terms of the GNU General Public License as published by the Free Software
-  Foundation; either version 2, or (at your option) any later version.
+  GNU ghostscript is free software; you can redistribute it and/or
+  modify it under the terms of the version 2 of the GNU General Public
+  License as published by the Free Software Foundation.
 
   GNU ghostscript is distributed in the hope that it will be useful, but WITHOUT
   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
@@ -14,10 +15,9 @@
   ghostscript; see the file COPYING. If not, write to the Free Software Foundation,
   Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 
-  
 */
 
-/* $Id: iscanbin.h,v 1.4 2005/12/13 16:57:26 jemarch Exp $ */
+/* $Id: iscanbin.h,v 1.6 2007/08/01 14:26:38 jemarch Exp $ */
 /* Interface to binary token scanner */
 
 #ifndef iscanbin_INCLUDED
@@ -32,7 +32,6 @@
  * this procedure: a dummy one for Level 1 systems, and the real one.
  * The interface is entirely internal to the scanner.
  */
-int scan_binary_token(i_ctx_t *i_ctx_p, stream *s, ref *pref,
-		      scanner_state *pstate);
+int scan_binary_token(i_ctx_t *i_ctx_p, ref *pref, scanner_state *pstate);
 
 #endif /* iscanbin_INCLUDED */
